﻿package 
{
	import com.divillysausages.ds.util.DSContextMenu;
	import flash.display.DisplayObject;
	import flash.display.MovieClip;
	import flash.display.Shape;
	import flash.events.Event;
	import flash.events.ProgressEvent;
	import flash.utils.getDefinitionByName;
	
	/**
	 * The Preloader loads the game and creates the Main class when we're ready
	 * @author Damian
	 */
	public class Preloader extends MovieClip 
	{
		private var m_bg:Shape			= null;
		private var m_bar:Shape 		= null;
		private var m_outline:Shape		= null;
		
		/**
		 * Creates our preloader; adds listeners to see if we're loaded yet
		 */
		public function Preloader() 
		{
			this.addEventListener( Event.ENTER_FRAME, this._checkFrame );
			this.loaderInfo.addEventListener( ProgressEvent.PROGRESS, this._onProgress );
			
			// create the bg
			this.m_bg = new Shape;
			this.m_bg.graphics.beginFill( 0x007100 );
			this.m_bg.graphics.drawRect( 0.0, 0.0, 800.0, 350.0 );
			this.m_bg.graphics.endFill();
			this.addChild( this.m_bg );
			
			// create our context menu
			var menu:DSContextMenu 	= new DSContextMenu;
			this.contextMenu 		= menu.menu;
			
			this._setupLoader();
		}
		
		/*******************************************************************************************/
		
		// setup the loader bar
		private function _setupLoader():void
		{			
			// create the bar
			this.m_bar = new Shape;
			this.m_bar.graphics.beginFill( 0xffffff );
			this.m_bar.graphics.drawRect( 0.0, 0.0, 100.0, 2.0 );
			this.m_bar.graphics.endFill();
			this.m_bar.x = 400.0 - this.m_bar.width * 0.5; 	// 400 = stage.stageWidth * 0.5
			this.m_bar.y = 175.0 - this.m_bar.height * 0.5;	// 175 = stage.stageHeight * 0.5
			this.addChild( this.m_bar );
			
			// the outline
			this.m_outline = new Shape;
			this.m_outline.graphics.lineStyle( 2.0, 0xffffff );
			this.m_outline.graphics.drawRect( 0.0, 0.0, 104.0, 6.0 );
			this.m_outline.x = this.m_bar.x - 2.0;
			this.m_outline.y = this.m_bar.y - 2.0;
			this.addChild( this.m_outline );
			
			this.m_bar.scaleX = 0.0;
		}
		
		// called as our loading progresses. update the preloader
		private function _onProgress( e:ProgressEvent ):void 
		{
			if ( this.m_bar == null )
				return;
				
			// update loader
			var pc:Number 		= e.bytesLoaded / e.bytesTotal;
			if ( pc > 1.0 )
				pc = 1.0;
			this.m_bar.scaleX 	= pc;
		}
		
		// checks every frame to see if we're fully loaded yet
		private function _checkFrame( e:Event ):void 
		{
			if ( this.currentFrame == this.totalFrames ) 
			{
				if( this.m_bar != null )
					this.m_bar.scaleX = 1.0;
				this.removeEventListener( Event.ENTER_FRAME, this._checkFrame );
				this._startup();
			}
		}
		
		// searches for the Main class, creates it, and adds it to the stage
		private function _startup():void 
		{
			// hide loader
			this.stop();
			this.loaderInfo.removeEventListener( ProgressEvent.PROGRESS, this._onProgress );
			var mainClass:Class = getDefinitionByName( "Main" ) as Class;
			this.stage.addChild( new mainClass() as DisplayObject );
			
			// clear the others
			this.removeChild( this.m_bg );
			this.m_bg = null;
			if ( this.m_bar != null )
			{
				this.removeChild( this.m_bar );
				this.removeChild( this.m_outline );
				this.m_bar 		= null;
				this.m_outline	= null;
			}
			
			// remove this
			if( this.parent != null )
				this.parent.removeChild( this );
		}
		
	}
	
}