package com.divillysausages.sprint.anim 
{
	import com.divillysausages.ds.collections.objectpool.IPoolObj;
	import com.divillysausages.ds.DS;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.DisplayObject;
	import flash.display.DisplayObjectContainer;
	import flash.display.MovieClip;
	import flash.geom.Point;
	
	/**
	 * The parameters that we use when playing animations
	 * @author Damian Connolly
	 */
	public class AnimParams implements IPoolObj
	{
		
		/*************************************************************************************************************/
		
		/**
		 * An ID that we want to give the anim, to help differentiate them
		 */
		public var id:String = null;
		
		/**
		 * The target for the animation. It should be either a MovieClip or a Bitmap object
		 */
		public var target:DisplayObject = null;
		
		/**
		 * The target class to create for this animation. Ignored if target is set
		 */
		public var targetClass:Class = null;
		
		/**
		 * The swf where we'll find our graphics, if we need to load them. If set, then
		 * instance can't be null
		 */
		public var swf:String = null;
		
		/**
		 * The instance name of our graphics. Needed if swf is set.
		 */
		public var instance:String = null;
		
		/**
		 * The parent to add the animation to
		 */
		public var parent:DisplayObjectContainer = null;
		
		/**
		 * The child index to add the object at
		 */
		public var childIndex:int = -1;
		
		/**
		 * The x position of the animation
		 */
		public var x:Number = NaN;
		
		/**
		 * The y position of the animation
		 */
		public var y:Number = NaN;
		
		/**
		 * The x scale of the animation
		 */
		public var scaleX:Number = NaN;
		
		/**
		 * The y scale of the animation
		 */
		public var scaleY:Number = NaN;
		
		/**
		 * The number of loops for this animation
		 */
		public var loops:int = 0;
		
		/**
		 * If we're looping to a specific place, this is either the int directly, or a String for a frame info for us to find
		 */
		public var loopTo:* = null;
		
		/**
		 * The tps speed for the animation
		 */
		public var tps:int = 0;
		
		/**
		 * The starting frame of the animation
		 */
		public var start:int = 0;
		
		/**
		 * The ending frame of the animation
		 */
		public var end:int = 0;
		
		/**
		 * The BitmapDatas to use if this is a bitmap animation
		 */
		public var bmds:Vector.<BitmapData> = null;
		
		/**
		 * Any offsets that we want to use for this animation
		 */
		public var bmdOffsets:Vector.<Point> = null;
		
		/**
		 * Either a String or an AnimFrameInfo object to get our start and end animation from. If it's
		 * a String, when we search for the right AnimFrameInfo on the animation when it's inited
		 */
		public var animFrameInfo:* /*AnimFrameInfo*/ = null;
		
		/**
		 * If we're looking for anim frames, and we want to specify an end tag (so anims go from
		 * start frame to start + endTag frame, this is it
		 */
		public var animFrameEndTag:String = null;
		
		/**
		 * When getting AnimFrameInfo objects, we stock them based on the class of the object that they're in. For
		 * the most part, this is fine, though sometimes we have different AnimFrameInfo objects under the same class
		 * (for example [object MovieClip]). This lets us specify our own key, if this conflict is likely to occur
		 */
		public var animFrameKey:String = null;
		
		/**
		 * The function to call when we init the animation, before anything is played. It can
		 * take at least one (optional) parameter of type Anim
		 */
		public var onInit:Function = null;
		
		/**
		 * Any additional parameters to pass to the onInit function. If the anim parameter is specified,
		 * these params are passed afterwards
		 */
		public var onInitParams:Array = null;
		
		/**
		 * The function to call every frame. It can take at least one (optional) parameter of type Anim
		 */
		public var onUpdate:Function = null;
		
		/**
		 * Any additional parameters to pass to the onUpdate function. If the anim parameter is specified,
		 * these params are passed afterwards
		 */
		public var onUpdateParams:Array = null;
		
		/**
		 * The function to call any time we pass a named frame. It can take at least two parameters;
		 * one (optional) of type Anim and one (compulsory) of type String (the frame). onFrame is ignored
		 * if the anim is an idle animation
		 */
		public var onFrame:Function = null;
		
		/**
		 * Any additional parameters to pass to the onFrame function. If the anim parameter is specified,
		 * these params are passed after it and the frame
		 */
		public var onFrameParams:Array = null;
		
		/**
		 * The function to call when we finish a loop. Called before onFinish as well. It can take
		 * at least one (optional) parameter of type Anim. onLoop is ignored if the anim is an idle
		 * animation
		 */
		public var onLoop:Function = null;
		
		/**
		 * Any additional parameters to pass to the onLoop function. If the anim parameter is specified,
		 * these params are passed after it
		 */
		public var onLoopParams:Array = null;
		
		/**
		 * The function to call when we finish all this animation. It can take at least one (optional) parameter
		 * of type Anim. onAnimFinish is ignored if the anim is an idle animation
		 */
		public var onAnimFinish:Function = null;
		
		/**
		 * Any additional parameters to pass to the onAnimFinish function. If the anim parameter is specified,
		 * these params are passed after it
		 */
		public var onAnimFinishParams:Array = null;
		
		/**
		 * The function to call when we finish the current chain of animations. This can either mean there's
		 * no more animations, or the next one is the idle animation. It can take at least one (optional) parameter
		 * of type Anim. onFinish is ignored if the anim is an idle animation
		 */
		public var onFinish:Function = null;
		
		/**
		 * Any additional parameters to pass to the onFinish function. If the anim parameter is specified,
		 * these params are passed after it
		 */
		public var onFinishParams:Array = null;
		
		/*************************************************************************************************************/
		
		/**
		 * Creates a new AnimParams object
		 * @param obj The object to get our parameters from
		 */
		public function AnimParams( obj:* = null ):void
		{
			if ( obj != null )
				this.fromObject( obj );
		}
		/**
		 * Init the initial params of the object
		 * @param params An optional parameters object
		 */
		public function init( params:* = null ):void
		{
			if ( params != null )
				this.fromObject( params );
		}
		
		/**
		 * Destroys the AnimParams and clears it for garbage collection
		 */
		public function destroy():void
		{
			// just call reset
			this.reset();
		}
		
		/**
		 * Resets the object so it can be reused
		 */
		public function reset():void
		{
			// reset the values
			this.x			= NaN;
			this.y			= NaN;
			this.scaleX		= NaN;
			this.scaleY		= NaN;
			this.loops		= 0;
			this.tps		= 0;
			this.start		= 0;
			this.end		= 0;
			this.childIndex	= -1;
			
			// null all our objects
			this.id					= null;
			this.target				= null;
			this.targetClass		= null;
			this.swf				= null;
			this.instance			= null;
			this.parent				= null;
			this.loopTo				= null;
			this.bmds				= null;
			this.bmdOffsets			= null;
			this.animFrameInfo		= null;
			this.animFrameEndTag	= null;
			this.animFrameKey		= null;
			this.onInit				= null;
			this.onInitParams		= null;
			this.onUpdate			= null;
			this.onUpdateParams		= null;
			this.onFrame			= null;
			this.onFrameParams		= null;
			this.onLoop				= null;
			this.onLoopParams		= null;
			this.onAnimFinish		= null;
			this.onAnimFinishParams	= null;
			this.onFinish			= null;
			this.onFinishParams		= null;
		}
		
		/**
		 * Fills our data from an object
		 * @param obj The object where we'll take all our data from
		 */
		public function fromObject( obj:* ):void
		{
			this._setParam( obj, "id", String );
			this._setParam( obj, "target", DisplayObject );
			this._setParam( obj, "targetClass", Class );
			this._setParam( obj, "swf", String );
			this._setParam( obj, "instance", String );
			this._setParam( obj, "parent", DisplayObjectContainer );
			this._setParam( obj, "childIndex", int );
			this._setParam( obj, "x", Number );
			this._setParam( obj, "y", Number );
			this._setParam( obj, "scaleX", Number );
			this._setParam( obj, "scaleY", Number );
			this._setParam( obj, "loops", int );
			this._setParam( obj, "loopTo", int, String );
			this._setParam( obj, "tps", int );
			this._setParam( obj, "start", int );
			this._setParam( obj, "end", int );
			this._setParam( obj, "bmds", Vector, BitmapData );
			this._setParam( obj, "bmdOffsets", Vector, Point );
			this._setParam( obj, "animFrameInfo", AnimFrameInfo, String );
			this._setParam( obj, "animFrameEndTag", String );
			this._setParam( obj, "animFrameKey", String );
			this._setParam( obj, "onInit", Function );
			this._setParam( obj, "onInitParams", Array );
			this._setParam( obj, "onUpdate", Function );
			this._setParam( obj, "onUpdateParams", Array );
			this._setParam( obj, "onFrame", Function );
			this._setParam( obj, "onFrameParams", Array );
			this._setParam( obj, "onLoop", Function );
			this._setParam( obj, "onLoopParams", Array );
			this._setParam( obj, "onAnimFinish", Function );
			this._setParam( obj, "onAnimFinishParams", Array );
			this._setParam( obj, "onFinish", Function );
			this._setParam( obj, "onFinishParams", Array );
		}
		
		/**
		 * Validates the info in the AnimParams object
		 * @return If the params are valid
		 */
		public function validate():Boolean
		{
			// we need either a target, a targetClass, or a swf + instance combo
			if ( this.target == null && this.targetClass == null && ( this.swf == null || this.instance == null ) )
			{
				DS.error( this, "We need either a target, a targetClass, or a swf + instance combo for the animation" );
				return false;
			}
			
			// if we have a target, then it has to be either a MovieClip or a Bitmap
			if ( this.target != null && !( this.target is MovieClip ) && !( this.target is Bitmap ) )
			{
				DS.error( this, "The target needs to be either a MovieClip or a Bitmap" );
				return false;
			}
			
			// if we don't have a target, targetClass and we only have a swf, then we need to have an instance and vice versa
			if ( this.target == null && this.targetClass == null && ( ( this.swf != null && this.instance == null ) || ( this.swf == null && this.instance != null ) ) )
			{
				DS.error( this, "Either the swf passed (" + this.swf + ") or the instance (" + this.instance + ") is null" );
				return false;
			}
			
			// if we have a loopTo, it needs to be either an int or a String
			if ( this.loopTo != null && !( this.loopTo is int ) && !( this.loopTo is String ) )
			{
				DS.error( this, "The loopTo parameter passed (" + this.loopTo + ") isn't an int for a String" );
				return false;
			}
			
			// if we have an animFrameInfo, it needs to be either an AnimFrameInfo object or a String
			if ( this.animFrameInfo != null && !( this.animFrameInfo is AnimFrameInfo ) && !( this.animFrameInfo is String ) )
			{
				DS.error( this, "The animFrameInfo parameter passed (" + this.animFrameInfo + ") isn't an AnimFrameInfo object or a String" );
				return false;
			}
			
			// the start and end can't be negative
			if ( this.start < 0 || this.end < 0 )
			{
				DS.error( this, "Either the start frame (" + this.start + ") or the end frame (" + this.end + ") is < 0" );
				return false;
			}
			
			// make sure the start is less than the end
			if ( this.end < this.start )
			{
				DS.warn( this, "Swapping the start and end frames as the start is (" + this.start + ") and the end is (" + this.end + ")" );
				var e:int 	= this.end;
				this.end	= this.start;
				this.start	= e;
			}
			
			// if this is a bitmap or a bitmap class, we need to have a bmds param
			if ( ( this.target != null && ( this.target is Bitmap ) ) || ( this.targetClass == Bitmap ) )
			{
				// make sure we have some bitmap datas
				if ( this.bmds == null )
				{
					DS.error( this, "A Bitmap animation requires a Vector of BitmapData objects in order to work" );
					return false;
				}
				
				// if we have some offsets, there needs to be the same amount
				if ( this.bmdOffsets != null && this.bmds.length != this.bmdOffsets.length )
				{
					DS.error( this, "The offsets passed (" + this.bmdOffsets.length + ") don't match the BitmapDatas passed (" + this.bmds.length + ")" );
					return false;
				}
			}
			
			// NOTE: for the callbacks, the anim property is optional, which is why we check for ( params ) and ( params + 1 )
			
			// make sure on init has enough parameters
			var params:int = ( this.onInitParams != null ) ? this.onInitParams.length : 0;
			if ( this.onInit != null && this.onInit.length != params && this.onInit.length != ( params + 1 ) )
			{
				DS.error( this, "The onInit function has " + this.onInit.length + " declared parameters, but we're passing " + params );
				return false;
			}
			
			// make sure on update has enough parameters
			params = ( this.onUpdateParams != null ) ? this.onUpdateParams.length : 0;
			if ( this.onUpdate != null && this.onUpdate.length != params && this.onUpdate.length != ( params + 1 ) )
			{
				DS.error( this, "The onUpdate function has " + this.onUpdate.length + " declared parameters, but we're passing " + params );
				return false;
			}
				
			// make sure on frame has enough parameters
			params = 1 + ( ( this.onFrameParams != null ) ? this.onFrameParams.length : 0 ); // +1 for the frame
			if ( this.onFrame != null && this.onFrame.length != params && this.onFrame.length != ( params + 1 ) )
			{
				DS.error( this, "The onFrame function has " + this.onFrame.length + " declared parameters, but we're passing " + params );
				return false;
			}
				
			// make sure on loop has enough parameters
			params = ( this.onLoopParams != null ) ? this.onLoopParams.length : 0;
			if ( this.onLoop != null && this.onLoop.length != params && this.onLoop.length != ( params + 1 ) )
			{
				DS.error( this, "The onLoop function has " + this.onLoop.length + " declared parameters, but we're passing " + params );
				return false;
			}
			
			// make sure on anim finish has enough parameters
			params = ( this.onAnimFinishParams != null ) ? this.onAnimFinishParams.length : 0;
			if ( this.onAnimFinish != null && this.onAnimFinish.length != params && this.onAnimFinish.length != ( params + 1 ) )
			{
				DS.error( this, "The onAnimFinish function has " + this.onAnimFinish.length + " declared parameters, but we're passing " + params );
				return false;
			}
				
			// make sure on finish has enough parameters
			params = ( this.onFinishParams != null ) ? this.onFinishParams.length : 0;
			if ( this.onFinish != null && this.onFinish.length != params && this.onFinish.length != ( params + 1 ) )
			{
				DS.error( this, "The onFinish function has " + this.onFinish.length + " declared parameters, but we're passing " + params );
				return false;
			}
				
			// it's passed
			return true;
		}
		
		/**
		 * Copies over common parameters if we don't have them set
		 * @param params The AnimParams that we're going to copy
		 */
		public function copyCommonParams( params:AnimParams ):void
		{
			// id
			if ( this.id == null )
				this.id = params.id;
				
			// parent
			if ( this.parent == null )
				this.parent = params.parent;
				
			// child index
			if ( this.childIndex == -1 )
				this.childIndex = params.childIndex;
			
			// x position
			if ( this._isNaN( this.x ) )
				this.x = params.x;
				
			// y position
			if ( this._isNaN( this.y ) )
				this.y = params.y;
				
			// scalex
			if ( this._isNaN( this.scaleX ) )
				this.scaleX = params.scaleX;
				
			// scaley
			if ( this._isNaN( this.scaleY ) )
				this.scaleY = params.scaleY;
				
			// on finish
			if ( this.onFinish == null )
			{
				this.onFinish 		= params.onFinish;
				this.onFinishParams = params.onFinishParams;
			}
		}
		
		/*************************************************************************************************************/
		
		// sets a parameter from an object
		private function _setParam( obj:*, name:String, type:Class, type2:Class = null ):void
		{
			// check that the param is there
			if ( !( name in obj ) )
				return;
				
			// make sure it's not null
			if ( obj[name] == null )
				return;
			
			// make sure it's the right type
			if ( type == Vector )
			{
				// special vector case
				if ( obj[name] is Vector.<*> )
				{
					// make sure it's the right type of vector
					if ( obj[name].length == 0 || !( obj[name][0] is type2 ) )
						DS.warn( this, "Found the '" + name + "' Vector, but it was empty or not of type " + type2 );
					else
						this[name] = obj[name]; // it's good set it
				}
				else
					DS.warn( this, "Found the '" + name + "' param, but it's not a Vector" );
			}
			else if ( obj[name] is type )
				this[name] = obj[name]; // it's good, set it
			else if ( type2 != null && obj[name] is type2 )
				this[name] = obj[name]; // it's good, set it
			else
			{
				var log:String = "Found the '" + name + "' param, but it's not a " + type;
				if ( type2 != null )
					log += " or a " + type;
				DS.warn( this, log );
			}
		}
		
		// helper function to determine if a number if NaN
		private function _isNaN( n:Number ):Boolean
		{
			// ( n != n ) will be true if the number is NaN
			return ( n != n );
		}
		
	}

}