package com.divillysausages.sprint.anim 
{
	import com.divillysausages.ds.asset.Asset;
	import com.divillysausages.ds.DS;
	import com.divillysausages.sprint.assets.Resources;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.system.System;
	import flash.utils.Dictionary;
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class AnimParser 
	{
		
		/*******************************************************************************************/
		
		private static var m_creating:Boolean		= false;
		private static var m_instance:AnimParser	= null;
		
		/*******************************************************************************************/
		
		/**
		 * Returns the singleton instance of the anim parser
		 */
		public static function get instance():AnimParser
		{
			if ( AnimParser.m_instance == null )
			{
				AnimParser.m_creating = true;
				AnimParser.m_instance = new AnimParser;
				AnimParser.m_creating = false;
			}
			return AnimParser.m_instance;
		}
		
		/*******************************************************************************************/
		
		private var m_anims:Dictionary 		= new Dictionary;	// all the anims that we extracted
		private var m_offsets:Dictionary	= new Dictionary;	// all the offsets for the anims
		private var m_frameInfos:Dictionary	= new Dictionary;	// all the frame infos
		
		/*******************************************************************************************/
		
		/**
		 * Creates a new AnimParser
		 */
		public function AnimParser() 
		{
			if ( !AnimParser.m_creating )
				throw new Error( "AnimParser is a singleton, use the instance static property instead" );
		}
		
		/**
		 * Inits the AnimParser and loads our xmls
		 */
		public function init():void
		{
			// get our xml and parse it
			var xml:Asset = DS.getAsset( Resources.RUNNER_XML );
			this._parseXML( xml );
		}
		
		/**
		 * Returns the frames for a specific animation
		 * @param anim The name of the animation that we're looking at
		 * @return A Vector of BitmapData, or null if we don't have ones
		 */
		public function getFrames( anim:String ):Vector.<BitmapData>
		{
			return this.m_anims[anim];
		}
		
		/**
		 * Returns the offsets for a specific animation
		 * @param anim The name of the animation that we're looking at
		 * @return A Vector of Point, or null if we don't have ones
		 */
		public function getOffsets( anim:String ):Vector.<Point>
		{
			return this.m_offsets[anim];
		}
		
		/**
		 * Returns the anim frame infos for a specific animation
		 * @param anim The name of the animation that we're looking at
		 * @return A Vector of AnimFrameInfo, or null if we don't have ones
		 */
		public function getAnimFrameInfos( anim:String ):Vector.<AnimFrameInfo>
		{
			return this.m_frameInfos[anim];
		}
		
		/*******************************************************************************************/
		
		// parsing the xml
		private function _parseXML( a:Asset ):void
		{
			var xml:XML			= a.xml;															// our xml
			var imageURL:String	= a.url.substring( 0, a.url.lastIndexOf( "/" ) + 1 ) + xml.@image;	// our image url
			var bmd:BitmapData 	= DS.getAsset( imageURL ).bitmapData;								// our ss image
			var w:int			= bmd.width; 														// ss width
			var fw:int			= int( xml.@frameW );												// frame width
			var fh:int			= int( xml.@frameH );												// frame height
			var ox:int			= int( xml.@offsetX );												// frame offset x
			var oy:int			= int( xml.@offsetY );												// frame offset y
			var cols:int		= w / fw;															// #images to a row
			var numFrames:int	= int( xml.@numFrames );											// total frames
			
			// create our offset point
			var offset:Point = new Point( ox, oy );
			
			// go through and extract all our frame bitmapdatas
			var v:Vector.<BitmapData> 	= new Vector.<BitmapData>;
			var rect:Rectangle			= new Rectangle( 0.0, 0.0, fw, fh );
			var point:Point				= new Point;
			for ( var i:int = 0; i < numFrames; i++ )
			{
				var b:BitmapData 	= new BitmapData( fw, fh, true, 0x00000000 );
				rect.x				= int( i % cols ) * fw;
				rect.y				= int( i / cols ) * fh;
				b.copyPixels( bmd, rect, point );
				v.push( b );
			}
			
			// now go through and get all our animations etc
			for each( var ax:XML in xml.animation )
			{
				var name:String 						= ax.@name.toString();
				var len:int								= ax.frame.length();
				var frames:Vector.<BitmapData>			= new Vector.<BitmapData>( len, true );
				var offsets:Vector.<Point>				= new Vector.<Point>( len, true );
				var frameInfos:Vector.<AnimFrameInfo>	= new Vector.<AnimFrameInfo>;
				i							 			= 0;
				for each( var fx:XML in ax.frame )
				{
					// find our frame and add our offset
					var index:int 	= int( fx.@index );
					frames[i]		= v[index];
					offsets[i]		= offset;
					
					// if we have any events, add them
					var events:String = fx.@events.toString();
					if ( events != null && events.length > 0 )
					{
						// split them, as there might be multiple
						var split:Array = events.split( "," );
						for ( var j:int = 0; j < split.length; j++ )
						{
							var info:AnimFrameInfo = new AnimFrameInfo( split[j], i, i );
							frameInfos.push( info );
						}
					}
					i++;
				}
					
				// add them all
				this.m_anims[name] 		= frames;
				this.m_offsets[name]	= offsets;
				this.m_frameInfos[name]	= frameInfos;
			}
			
			// we don't need our vector, bitmap, or xml anymore, so free the memory
			v.length = 0;
			bmd.dispose();
			System.disposeXML( xml );
		}
		
	}

}