package com.divillysausages.sprint.gui 
{
	import com.divillysausages.sprint.assets.Resources;
	import com.divillysausages.sprint.constants.Style;
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	
	/**
	 * The GameButton for us
	 * @author Damian
	 */
	public class GameButton extends Sprite
	{
		
		/*******************************************************************************************/
		
		{
			// static ctor - create our textformat
			m_format = new TextFormat( Resources.GAME_FONT, Style.BUTTON_SIZE, Style.BUTTON_COLOUR );
		}
		
		/*******************************************************************************************/
		
		private static var m_format:TextFormat = null; // the textformat that we all use
		
		/*******************************************************************************************/
		
		private var m_width:Number			= 150.0;// the width of the button
		private var m_height:Number			= 50.0;	// the height of the button
		private var m_text:TextField 		= null;	// the text for this button
		private var m_highlighted:Boolean	= false;// is this button highlighted?
		private var m_pressed:Boolean		= false;// is this button pressed?
		private var m_pressOffset:Number	= 3.0;	// how much to move when we're pressed
		private var m_onClick:Function		= null;	// the callback to call when we're clicked
		
		/*******************************************************************************************/
		
		/**
		 * [write-only] Set if the button's active
		 */
		public function set active( b:Boolean ):void
		{
			if ( b )
				this._activate();
			else
				this._deactivate();
		}
		
		// set if we're pressed or not
		private function set pressed( b:Boolean ):void
		{
			this.m_pressed = b;
			
			// set the text y
			this.m_text.y = int( this.m_height * 0.5 - this.m_text.height * 0.5 - 2.0 ); // -2 as the text is a bit off
			if ( this.m_pressed )
				this.m_text.y += int( this.m_pressOffset );
		}
		
		/**
		 * The x value for the game button
		 */
		override public function set x( n:Number ):void 
		{
			super.x = int( n );
		}
		
		/**
		 * The y value for the game button
		 */
		override public function set y( n:Number ):void 
		{
			super.y = int( n );
		}
		
		/*******************************************************************************************/
		
		/**
		 * Creates our GameButton
		 */
		public function GameButton( label:String, onClick:Function ) 
		{
			// create our text
			this._createText( label.toUpperCase() );
			
			// hold our onClick
			this.m_onClick = onClick;
			
			// set our mouse props
			this.buttonMode 	= true;
			this.useHandCursor	= true;
			
			// draw our initial button
			this._draw();
		}
		
		public function destroy():void
		{
			this._deactivate();
			
			// remove graphics
			this.removeChildren();
			if ( this.parent != null )
				this.parent.removeChild( this );
				
			// null our propertie
			this.m_text		= null;
			this.m_onClick	= null;
		}
		
		/*******************************************************************************************/
		
		// creates the text for this button
		private function _createText( label:String ):void
		{
			this.m_text 					= new TextField;
			this.m_text.defaultTextFormat	= GameButton.m_format;
			this.m_text.text				= label;
			this.m_text.autoSize			= TextFieldAutoSize.LEFT;
			this.m_text.mouseEnabled		= false;
			this.m_text.mouseWheelEnabled	= false;
			this.m_text.selectable			= false;
			this.m_text.embedFonts			= true;
			this.m_text.x					= int( this.m_width * 0.5 - this.m_text.width * 0.5 );
			this.m_text.y					= int( this.m_height * 0.5 - this.m_text.height * 0.5 - 2.0 ); // -2 as the text is a bit off
			
			// add it
			this.addChild( this.m_text );
		}
		
		// draw our button
		private function _draw():void
		{
			// clear our old graphics
			this.graphics.clear();
			
			// get our width and height
			var w:Number 		= this.m_width;		// the width of the button
			var h:Number 		= this.m_height;	// the height of the button
			var shadowX:Number 	= 7.0;				// the shadow offset
			var shadowY:Number 	= 10.0;				// the shadow offset
			var corner:Number	= 20.0;				// the corner size
			var outline:Number	= 5.0;				// how thick the outline is
			var offset:Number	= ( this.m_pressed ) ? this.m_pressOffset : 0.0;	// how much offset to add to the button
			
			var shadowCol:uint 	= 0x002000;	// the colour of our shadow
			var outlineCol:uint	= 0x035daa;	// the colour of our outline
			var fillCol:uint	= ( this.m_pressed ) ? 0x0966b4 : ( this.m_highlighted ) ? 0x2696f5 : 0x0181ee; // our fill colour
			
			// draw our shadow
			this.graphics.beginFill( shadowCol );
			this.graphics.moveTo( shadowX + corner, shadowY );
			this.graphics.lineTo( shadowX + w, shadowY );
			this.graphics.lineTo( shadowX + w, shadowY + h - corner );
			this.graphics.lineTo( shadowX + w - corner, shadowY + h );
			this.graphics.lineTo( shadowX, shadowY + h );
			this.graphics.lineTo( shadowX, shadowY + corner );
			this.graphics.lineTo( shadowX + corner, shadowY );
			
			// draw our outline
			this.graphics.beginFill( outlineCol );
			this.graphics.moveTo( -outline + corner, -outline + offset );
			this.graphics.lineTo( outline + w, -outline + offset );
			this.graphics.lineTo( outline + w, outline + h - corner + offset );
			this.graphics.lineTo( outline + w - corner, outline + h + offset );
			this.graphics.lineTo( -outline, outline + h + offset );
			this.graphics.lineTo( -outline, -outline + corner + offset );
			this.graphics.lineTo( -outline + corner, -outline + offset );
			
			// draw our button bg
			this.graphics.beginFill( fillCol );
			this.graphics.moveTo( corner, offset );
			this.graphics.lineTo( w, offset );
			this.graphics.lineTo( w, h - corner + offset );
			this.graphics.lineTo( w - corner, h + offset );
			this.graphics.lineTo( 0.0, h + offset );
			this.graphics.lineTo( 0.0, corner + offset );
			this.graphics.lineTo( corner, offset );
		}
		
		// activates the button by adding all the event listeners
		private function _activate():void
		{
			this.addEventListener( MouseEvent.MOUSE_OVER, this._onOver );
			this.addEventListener( MouseEvent.MOUSE_OUT, this._onOut );
			this.addEventListener( MouseEvent.MOUSE_DOWN, this._onDown );
			this.addEventListener( MouseEvent.MOUSE_UP, this._onUp );
			this.addEventListener( MouseEvent.CLICK, this._onClick );
		}
		
		// deactivates the button by removing all the event listeners
		private function _deactivate():void
		{
			this.removeEventListener( MouseEvent.MOUSE_OVER, this._onOver );
			this.removeEventListener( MouseEvent.MOUSE_OUT, this._onOut );
			this.removeEventListener( MouseEvent.MOUSE_DOWN, this._onDown );
			this.removeEventListener( MouseEvent.MOUSE_UP, this._onUp );
			this.removeEventListener( MouseEvent.CLICK, this._onClick );
			
			// make sure we're not in an active state
			this.m_highlighted 	= false;
			this.pressed		= false;
			this._draw();
		}
		
		/*******************************************************************************************/
		
		// highlights the button
		private function _onOver( e:MouseEvent ):void
		{
			this.m_highlighted 	= true;
			this.pressed		= false;
			this._draw();
		}
		
		// stops highlighting the button
		private function _onOut( e:MouseEvent ):void
		{
			this.m_highlighted	= false;
			this.pressed		= false;
			this._draw();
		}
		
		// presses the button
		private function _onDown( e:MouseEvent ):void
		{
			this.pressed = true;
			this._draw();
		}
		
		// releases the button
		private function _onUp( e:MouseEvent ):void
		{
			this.pressed = false;			
			this._draw();
		}
		
		// clicks the button; call our callback
		private function _onClick( e:MouseEvent ):void
		{
			if ( this.m_onClick != null )
				this.m_onClick();
		}
		
	}

}