package com.divillysausages.sprint.gui
{
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.security.SafeNumber;
	import com.divillysausages.ds.util.Colour;
	import com.divillysausages.sprint.assets.Resources;
	import com.divillysausages.sprint.constants.Constants;
	import com.divillysausages.sprint.constants.Style;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.geom.ColorTransform;
	import flash.geom.Matrix;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	
	public class GameTimer extends Sprite implements IUpdateObj
	{
		
		/*******************************************************************************************/
		
		private static const DOT_INDEX:int 		= 10;	// the bmd index for the dot
		private static const BLANK_INDEX:int	= 11;	// the bmd index for the blank
		
		/*******************************************************************************************/
		
		protected var m_bmds:Vector.<BitmapData>	= new Vector.<BitmapData>( 12, true );	// our number bmds (0-9, ., blank)
		protected var m_bitmaps:Vector.<Bitmap>		= new Vector.<Bitmap>( 6, true );		// 00.000
		protected var m_time:SafeNumber 			= new SafeNumber( 0 );					// our actual time
		protected var m_isActive:Boolean			= false;								// have we started?
		protected var m_updateDisplay:Boolean		= true;									// should we update the display?
		
		/*******************************************************************************************/
		
		/**
		 * Returns the time on the timer, as a string
		 */
		public function get time():String 
		{
			var n:Number	= this.m_time.value;
			var str:String 	= n.toFixed( 3 );
			return ( n < 10 ) ? "0" + str : str;
		}
		
		/**
		 * The time on the timer, as a number
		 */
		public function get timeNumber():Number { return this.m_time.value; }
		
		/**
		 * If we're active or not (for the update)
		 */
		public function get active():Boolean { return this.m_isActive; }
		
		/*******************************************************************************************/
		
		/**
		 * Creates a new GameTimer
		 */
		public function GameTimer()
		{
			// create our text
			var t:TextField		= new TextField;
			t.defaultTextFormat	= new TextFormat( Resources.GAME_FONT, Style.GAME_TIMER_SIZE, Style.GAME_TIME_COLOUR );
			t.autoSize			= TextFieldAutoSize.LEFT;
			t.selectable		= false;
			t.mouseEnabled		= false;
			t.mouseWheelEnabled	= false;
			t.multiline			= false;
			t.embedFonts		= true;
			t.text				= "0";
			
			// get the max width and height to our next series of pixel sizes
			var ps:int	= Constants.PIXEL_SIZE;
			var w:int 	= Math.ceil( t.width / ps ) * ps;
			var h:int 	= Math.ceil( t.height / ps ) * ps - ps * 5; // - ps * 5 to remove the textfield vertical space
			
			// go through and generate all our bmds
			var c:Colour 			= Colour.fromBlend( Colour.BLACK, Colour.WHITE, 0.2 );
			var m:Matrix			= new Matrix;
			var ct:ColorTransform	= new ColorTransform( 1, 1, 1, 0.6 );
			for ( var i:int = 0; i < 12; i++ )
			{
				// create our bmd
				var bmd:BitmapData = new BitmapData( w, h, true, c.argb );
				
				// NOTE: the math here is to try and center the number with a border of PIXEL_SIZE around
				// it. Due to the textfield padding etc, this isn't entirely possible, hence the +2, +1 etc
				// that you see, as it's corrected by eye. As such, it'll need to be redone if the PIXEL_SIZE,
				// text size or font change
				
				// numbers
				if ( i < 10 )
				{
					t.text = i + "";
					m.identity();
					
					// move to center it
					var n:Number 	= ( w * 0.5 - t.width * 0.5 ) / ps;
					var x:int		= int( Math.ceil( n ) * ps );
					m.translate( x + 2, -ps * 4 + 2 );
				}
				else if( i < 11 ) // dot
				{
					t.text = ".";
					m.identity();
					m.translate( ps * 2 - 2, -ps * 4 + 1 ); // center the dot
				}
				
				// draw it
				if( i < 11 )
					bmd.draw( t, m, ct );
				
				// add it to our vector
				this.m_bmds[i] = bmd;
			}
			
			// create all our bitmaps
			for ( i = 0; i < 6; i++ )
			{
				var b:Bitmap 		= new Bitmap( this.m_bmds[GameTimer.BLANK_INDEX] );
				b.x			 		= ps * 2 + ( bmd.width + ps ) * i;
				b.y					= ps * 2;
				this.m_bitmaps[i]	= b;
				this.addChild( b );
			}
			
			// draw our bg
			this._drawBG();
			
			// set our position
			this.x = int( DS.stage.stageWidth * 0.5 - this.width * 0.5 );
			this.y = 15;
			
			// add us to the update
			DS.addToUpdate( this );
			
			// stop interaction
			this.mouseChildren	= false;
			this.mouseEnabled	= false;
			
			// reset us to start
			this.reset();
		}
		
		/**
		 * Destroys the GameTimer and clears it for garbage collection
		 */
		public function destroy():void
		{
			// remove from update
			DS.removeFromUpdate( this );
			
			// graphics
			if ( this.parent != null )
				this.parent.removeChild( this );
				
			// null our values
			this.m_time	= null;
		}
		
		/**
		 * Resets the GameTimer
		 */
		public function reset():void
		{
			this.m_time.value		= 0.0;
			this.m_isActive			= false;
			this.m_updateDisplay	= true;
			this._updateTimeDisplay();
		}
		
		/**
		 * Starts the game timer
		 */
		public function start():void
		{
			this.m_isActive = true;
		}
		
		/**
		 * Stops the timer without resetting it
		 */
		public function stop():void
		{
			this.m_isActive = false;
		}
		
		/**
		 * Stops updating the display while keeping the clock going
		 */
		public function stopDisplay():void
		{
			this.m_updateDisplay = false;
		}
		
		/**
		 * Called every update for the timer
		 * @param dt The delta time
		 */
		public function update(dt:Number):void 
		{
			this.m_time.value += dt;
			this._updateTimeDisplay();
		}
		
		/*******************************************************************************************/
		
		// draw our bg
		protected function _drawBG():void
		{
			// calculate our vars
			var ps:int 		= Constants.PIXEL_SIZE;
			var len:int		= this.m_bitmaps.length;
			var w:int		= ( this.m_bmds[0].width + ps ) * len - ps;
			var h:int		= this.m_bmds[0].height;
			var c:Colour	= Colour.fromHexi( Style.GAME_TIME_COLOUR );
			c				= Colour.fromBlend( c, Colour.BLACK, 0.1 );
			
			// draw our bg
			this.graphics.beginFill( c.rgb );
			this.graphics.drawRect( ps, ps, w + ps * 2, h + ps * 2 );
			this.graphics.endFill();
			
			// highlight
			var hc:Colour = Colour.fromBlend( c, Colour.WHITE, 0.4 );
			this.graphics.beginFill( hc.rgb );
			this.graphics.drawRect( ps, 0.0, w + ps * 2, ps ); // hor
			this.graphics.endFill();
			this.graphics.beginFill( hc.rgb );
			this.graphics.drawRect( 0.0, ps, ps, h + ps * 2 ); // vert
			this.graphics.endFill();
			
			// shadow
			var sc:Colour = Colour.fromBlend( c, Colour.BLACK, 0.4 );
			this.graphics.beginFill( sc.rgb );
			this.graphics.drawRect( ps, h + ps * 3, w + ps * 2, ps ); // hor
			this.graphics.endFill();
			this.graphics.beginFill( sc.rgb );
			this.graphics.drawRect( w + ps * 3, ps, ps, h + ps * 2 ); // vert
			this.graphics.endFill();
		}
		
		// updates the time display
		protected function _updateTimeDisplay():void
		{
			if ( !this.m_updateDisplay )
				return;
				
			// get the time as a string
			var str:String = this.time;
				
			// now go through each digit in the string and set the values
			var len:int = str.length;
			if ( len >= this.m_bitmaps.length )
				len = this.m_bitmaps.length;
			for ( var i:int = 0; i < len; i++ )
			{
				var s:String = str.charAt( i );
				if ( s == "." )
					this.m_bitmaps[i].bitmapData = this.m_bmds[GameTimer.DOT_INDEX];
				else
					this.m_bitmaps[i].bitmapData = this.m_bmds[int( s )];
			}
		}
		
	}

}