package com.divillysausages.sprint.gui
{
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.security.SafeNumber;
	import com.divillysausages.ds.util.Colour;
	import com.divillysausages.sprint.constants.Constants;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.geom.Point;
	import org.osflash.signals.Signal;
	
	/**
	 * The gauge that shows the runner's efforts
	 * @author Damian
	 */
	public class Gauge extends Sprite
	{
		
		/*******************************************************************************************/
		
		private static const BAR_HEIGHT:Number 	= Constants.PIXEL_SIZE * 20;
		private static const BAR_WIDTH:Number 	= Constants.PIXEL_SIZE * 2;
		
		/*******************************************************************************************/
		
		/**
		 * The signal dispatched when we're over our limit. It should take no parameters
		 */
		public var signalOnOverLimit:Signal = null;
		
		/*******************************************************************************************/
		
		private var m_val:SafeNumber 	= new SafeNumber( 0.0 );	// the value of our gauge
		private var m_cutoff:Number 	= 0.9;						// the cutoff % before we're injured				
		private var m_framesOver:int 	= 0;						// the number of frames that we're over the cutoff, to give some leeway
		private var m_bar:Shape	 		= null;						// the inner bar
		
		/*******************************************************************************************/
		
		/**
		 * The value for the gauge, between 0.0 and 1.0
		 */
		public function get value():Number { return this.m_val.value; }
		public function set value( n:Number ):void
		{
			n 					= ( n < 0.0 ) ? 0.0 : ( n > 1.0 ) ? 1.0 : n;
			this.m_val.value	= n;
			this.m_bar.scaleY	= n;
			
			// check if we're over the limit
			if ( this.m_val.value > this.m_cutoff && this.m_framesOver > 2 )
				this.signalOnOverLimit.dispatch();
			else if ( this.m_val.value > this.m_cutoff )
			{
				DS.debug( this, "Over the limit, curr frames: " + this.m_framesOver );
				this.m_framesOver++;
			}
			else
				this.m_framesOver = 0;
		}
		
		/*******************************************************************************************/
		
		/**
		 * Creates a new gauge
		 */
		public function Gauge()
		{
			// create our signal
			this.signalOnOverLimit = new Signal;
			
			// draw our graphics
			var ps:int 		= Constants.PIXEL_SIZE;
			var bw:int 		= Gauge.BAR_WIDTH;
			var bh:int 		= Gauge.BAR_HEIGHT;
			var c:Colour	= Colour.fromBlend( Colour.BLACK, Colour.WHITE, 0.2 );
			
			// top
			this.graphics.beginFill( c.rgb );
			this.graphics.drawRect( ps, 0.0, bw, ps );
			this.graphics.endFill();
			
			// left
			this.graphics.beginFill( c.rgb );
			this.graphics.drawRect( 0.0, ps, ps, bh );
			this.graphics.endFill();
			
			// right
			this.graphics.beginFill( 0x0 );
			this.graphics.drawRect( ps + bw, ps, ps, bh );
			this.graphics.endFill();
			
			// bottom
			this.graphics.beginFill( 0x0 );
			this.graphics.drawRect( ps, ps + bh, bw, ps );
			this.graphics.endFill();
			
			// good
			c = Colour.fromBlend( Colour.BLUE, Colour.BLACK, 0.4 );
			this.graphics.beginFill( c.rgb );
			this.graphics.drawRect( ps, ps, bw, bh );
			this.graphics.endFill();
			
			// bad
			c = Colour.fromBlend( Colour.RED, Colour.BLACK, 0.4 );
			this.graphics.beginFill( c.rgb );
			this.graphics.drawRect( ps, ps, bw, bh * ( 1.0 - this.m_cutoff ) );
			this.graphics.endFill();
			
			
			// create our bar
			c			= Colour.fromBlend( Colour.GREEN, Colour.BLACK, 0.4 );
			this.m_bar 	= new Shape;
			this.m_bar.graphics.beginFill( c.rgb );
			this.m_bar.graphics.drawRect( 0.0, 0.0, bw, -bh );
			this.m_bar.graphics.endFill();
			
			// position and add it
			this.m_bar.x = ps;
			this.m_bar.y = ps + bh;
			this.addChild( this.m_bar );
		}
		
		/**
		 * Destroys the gauge and clears it for garbage collection
		 */
		public function destroy():void
		{
			// clear our signal listeners
			this.signalOnOverLimit.removeAll();
			
			// kill our graphics
			this.removeChild( this.m_bar );
			if ( this.parent != null )
				this.parent.removeChild( this );
				
			// null our properties
			this.signalOnOverLimit	= null;
			this.m_bar				= null;
			this.m_val				= null;
		}
		
	}

}