package com.divillysausages.sprint.gui
{
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.sprint.assets.Resources;
	import com.divillysausages.sprint.constants.Style;
	import flash.filters.DropShadowFilter;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	import org.osflash.signals.Signal;
	
	public class StartCountdown extends TextField implements IUpdateObj
	{	
		
		/*******************************************************************************************/
		
		/**
		 * The text for the ready state
		 */
		public static const READY:String = "On your marks...";
		
		/**
		 * The text for the set state
		 */
		public static const SET:String = "Get set...";
		
		/**
		 * The text for the go state
		 */
		public static const GO:String = "GO!";
		
		/*******************************************************************************************/
		
		/**
		 * The signal dispatched on ready. It should take no parameters
		 */
		public var signalOnReady:Signal = new Signal;
		
		/**
		 * The signal dispatched on set. It should take no parameters
		 */
		public var signalOnSet:Signal = new Signal;
		
		/**
		 * The signal dispatched on go. It should take no parameters
		 */
		public var signalOnGo:Signal = new Signal;
		
		/*******************************************************************************************/
		
		private var m_time:Number 		= 2.0;	// the total start time
		private var m_currTime:Number	= 0.0;	// the current time
		private var m_prevTime:int		= 0;	// the previous time (int)
		private var m_isActive:Boolean	= false;// are we active?
		
		/*******************************************************************************************/
		
		public function get active():Boolean { return this.m_isActive; }
		
		/*******************************************************************************************/
		
		/**
		 * Creates a new StartCountdown object
		 */
		public function StartCountdown():void
		{
			// graphics
			this.defaultTextFormat 	= new TextFormat( Resources.GAME_FONT, Style.COUNTDOWN_SIZE, Style.COUNTDOWN_COLOUR_NORMAL );
			this.autoSize			= TextFieldAutoSize.LEFT;
			this.selectable			= false;
			this.mouseEnabled		= false;
			this.mouseWheelEnabled	= false;
			this.multiline			= false;
			this.embedFonts			= true;
			
			var shadow:DropShadowFilter = new DropShadowFilter( 0, 0, 0xffffff, 1, 4, 4, 40 );
			this.filters 				= [shadow];
			
			// add us to the update
			DS.addToUpdate( this );
			
			// reset to begin
			this.reset();
		}
		
		/**
		 * Destroys the StartCountdown and clears it for garbage collection
		 */
		public function destroy():void
		{
			DS.removeFromUpdate( this );
			
			// graphics
			if ( this.parent != null )
				this.parent.removeChild( this );
		}
		
		/**
		 * Resets the countdown
		 */
		public function reset():void
		{
			this.m_isActive = false;
			this.visible	= false;
		}
		
		/**
		 * Starts the countdown
		 */
		public function start():void
		{
			this.m_currTime = this.m_time;
			this.m_prevTime	= int( this.m_time );
			this.alpha		= 1.0;
			this.visible	= true;
			this.m_isActive	= true;
			
			// set our text
			this.textColor = Style.COUNTDOWN_COLOUR_NORMAL;
			this._setText( StartCountdown.READY );
			this.signalOnReady.dispatch();
		}
		
		/**
		 * Updates the StartCountdown
		 * @param dt The delta time
		 */
		public function update( dt:Number ):void
		{
			this.m_currTime -= dt;
			var time:int	= int( this.m_currTime + 1.0 );
			
			// check if we need to set off an event
			if ( time != this.m_prevTime )
			{
				if ( time == 1 )
				{
					this._setText( StartCountdown.SET );
					this.signalOnSet.dispatch();
				}
				else
				{
					this.textColor = Style.COUNTDOWN_COLOUR_GO;
					this._setText( StartCountdown.GO );
					this.signalOnGo.dispatch();
				}
				this.m_prevTime = time;
			}
			
			// fade us out if necessary
			if ( this.m_currTime < 0.0 )
			{
				this.alpha -= dt;
				if ( this.alpha <= 0.0 )
				{
					this.visible 	= false;
					this.m_isActive	= false;
				}
			}
		}
		
		/*******************************************************************************************/
		
		// sets the text on the display
		private function _setText( s:String ):void
		{
			this.text 	= s;
			this.x		= int( DS.stage.stageWidth * 0.5 - this.width * 0.5 );
			this.y		= int( DS.stage.stageHeight * 0.5 - this.height * 0.5 );
		}
		
	}

}