package com.divillysausages.sprint.logic 
{
	import com.divillysausages.ds.cache.LSO;
	import com.divillysausages.ds.input.Input;
	import com.divillysausages.ds.input.Key;
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class GameSave 
	{
		
		/*******************************************************************************************/
		
		private static var m_creating:Boolean 	= false;
		private static var m_instance:GameSave	= null;
		
		/*******************************************************************************************/
		
		public static function get instance():GameSave
		{
			if ( GameSave.m_instance == null )
			{
				GameSave.m_creating = true;
				GameSave.m_instance = new GameSave;
				GameSave.m_creating = false;
			}
			return GameSave.m_instance;
		}
		
		/*******************************************************************************************/
		
		private var m_lso:LSO = null; // the lso that saves our data
		
		/*******************************************************************************************/
		
		/**
		 * Returns the current world record
		 * @return The world record
		 */
		public function get worldRecord():Number
		{
			var n:* = this.m_lso.get( "WR" );
			if ( n == null || !( n is Number ) || isNaN( n ) )
				return Number.MAX_VALUE;
			return n as Number;
		}
		public function set worldRecord( n:Number ):void
		{
			this.m_lso.add( "WR", n, false, true );
		}
		
		/*******************************************************************************************/
		
		public function GameSave() 
		{
			if ( !GameSave.m_creating )
				throw new Error( "GameSave is a singleton - use the static instance property instead" );
				
			// get our lso
			this.m_lso = new LSO( "GameSave" );
		}
		
		/**
		 * Resets the stats
		 */
		public function reset():void
		{
			this.m_lso.destroy();
		}
		
		/**
		 * Gets the personal best for a specific player
		 * @param playerNum The number of the player that we're looking for
		 * @return The player's personal best
		 */
		public function getPBForPlayer( playerNum:int ):Number
		{
			var n:* = this.m_lso.get( "PB" + playerNum );
			if ( n == null || !( n is Number ) || isNaN( n ) )
				return Number.MAX_VALUE;
			return n as Number;
		}
		
		/**
		 * Sets the personal best for a player
		 * @param playerNum The player number that we're looking at
		 * @param pb The personal best for the player
		 */
		public function setPBForPlayer( playerNum:int, pb:Number ):void
		{
			this.m_lso.add( "PB" + playerNum, pb, false, true );
		}
		
	}

}