package com.divillysausages.sprint.logic
{	
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.input.Input;
	import com.divillysausages.ds.input.Key;
	import com.divillysausages.sprint.assets.Resources;
	import com.divillysausages.sprint.constants.Anims;
	import com.divillysausages.sprint.constants.Style;
	import com.divillysausages.sprint.gui.GameInstructions;
	import com.divillysausages.sprint.gui.GameTimer;
	import com.divillysausages.sprint.gui.PostToTwitterButton;
	import com.divillysausages.sprint.gui.SaveImageButton;
	import com.divillysausages.sprint.gui.StartCountdown;
	import flash.filters.DropShadowFilter;
	import flash.geom.Point;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	
	public class Logic implements IUpdateObj
	{
		
		/*******************************************************************************************/
		
		private var m_timer:GameTimer 					= null;	// the main game timer
		private var m_countdown:StartCountdown 			= null;	// the start countdown
		private var m_gameInstructions:GameInstructions	= null; // the game instructions
		private var m_btnSaveImage:SaveImageButton		= null; // the save image button
		private var m_btnTwitter:PostToTwitterButton	= null; // the post to twitter button
		private var m_isGameStarted:Boolean 			= false;// is the game started?
		private var m_isGameStopped:Boolean				= false;// is the game stopped?
		private var m_hasGameJustStopped:Boolean 		= false;// has the game just stopped
		private var m_waitTime:int 						= 2000;	// how long to wait before restarting
		private var m_numRunners:int 					= 4;	// the number of runners in the race
		private var m_numRunnersFinished:int 			= 0;	// the number of runners finished
		private var m_tracks:Vector.<Track> 			= new Vector.<Track>( m_numRunners, true );
		private var m_lastPosX:Number 					= 0.0;	// last position (used for the winning line)
		private var m_winningLineStartX:Number 			= 0.0;	// the winning line start x
		private var m_numHumans:int 					= 0;	// the number of humans in the game
		private var m_numHumansInjured:int 				= 0;	// the number of humans injured in the game
		private var m_numHumansFinished:int 			= 0;	// the number of humans that have finished the game
		private var m_firstHumanFinisherTrack:Track 	= null;	// the track of the first human to finish
		private var m_firstHumanFinisherNumber:String 	= null;	// the number of the first human to finish
		private var m_firstHumanFinisherPos:int 		= 0;	// the position of the first human to finish
		
		/*******************************************************************************************/
		
		/**
		 * Is the logic active?
		 */
		public function get active():Boolean { return true; }
		
		/**
		 * The current time, as a string
		 */
		public function get time():String { return this.m_timer.time; }
		
		/**
		 * The current time, as a number
		 */
		public function get timeNumber():Number { return this.m_timer.timeNumber; }
		
		/**
		 * The number of runners in the race
		 */
		public function get numRunners():int { return this.m_numRunners; }
		
		/**
		 * Our post to twitter button
		 */
		public function get btnTwitter():PostToTwitterButton { return this.m_btnTwitter; }
		
		/*******************************************************************************************/
		
		/**
		 * Creates a new Logic objecct
		 * @param numHumans The number of humans in the races
		 */
		public function Logic( numHumans:int = 1 ):void
		{
			this.m_numHumans = numHumans;
		}
		
		/**
		 * Inits the logic
		 */
		public function init():void
		{		
			// create the track
			for ( var i:int = 0; i < this.m_numRunners; i++ )
			{
				// create our track and add our signals
				var t:Track = new Track( this, i );
				t.signalOnRunnerFalseStart.add( this._onRunnerFalseStarted );
				t.signalOnRunnerFinished.add( this._onRunnerFinished );
				t.signalOnRunnerInjured.add( this._onRunnerInjured );
				
				// create our runner
				t.createRunner();
				if ( i == 1 )
				{
					t.runner.isAI 			= false;
					t.runner.controlKey		= Key.SPACE;
					//t.runner.playerNumber	= "P1";
				}
				else if ( i == 2 && this.m_numHumans > 1 )
				{
					t.runner.isAI			= false;
					t.runner.controlKey		= Key.CONTROL;
					//t.runner.playerNumber	= "P2";
				}
				else
					t.runner.isAI = true;
				DS.canvas.addChild( t );
				this.m_tracks[i] = t;
			}
			
			// add the winning line
			DS.canvas.addChild( Track.winningLine );
			
			// add all the runners
			for ( i = this.m_tracks.length - 1; i >= 0; i-- )
				DS.canvas.addChild( this.m_tracks[i].runner );
			
			// the winning line start x
			var tw:Number 				= this.m_tracks[0].width;
			this.m_winningLineStartX 	= 50.0 + ( this.m_numRunners / 2 ) * tw - ( tw * 0.5 );
			
			// create the timer
			this.m_timer = new GameTimer;
			DS.canvas.addChild( this.m_timer );
			
			// create the countdown
			this.m_countdown = new StartCountdown;
			this.m_countdown.signalOnReady.add( this._onCountdownReady );
			this.m_countdown.signalOnSet.add( this._onCountdownSet );
			this.m_countdown.signalOnGo.add( this._onCountdownGo );
			DS.canvas.addChild( this.m_countdown );
			
			// add our instructions
			var instructions:TextField 		= new TextField;
			instructions.defaultTextFormat	= new TextFormat( Resources.GAME_FONT, Style.INSTRUCTIONS_SIZE, Style.INSTRUCTIONS_COLOUR );
			instructions.autoSize			= TextFieldAutoSize.LEFT;
			instructions.mouseEnabled		= false;
			instructions.mouseWheelEnabled	= false;
			instructions.selectable			= false;
			instructions.multiline			= false;
			instructions.embedFonts			= true;
			instructions.text				= "Press SPACE to power up, let go to power down";
			instructions.x					= DS.stage.stageWidth * 0.5 - instructions.width * 0.5;
			instructions.y					= DS.stage.stageHeight - instructions.height - 10;
			instructions.filters			= [new DropShadowFilter( 0, 0, 0, 1, 4, 4, 40 )];
			DS.canvas.addChild( instructions );
			
			// the game instructions
			this.m_gameInstructions 	= new GameInstructions;
			this.m_gameInstructions.y	= instructions.y - this.m_gameInstructions.height - 4;
			DS.canvas.addChild( this.m_gameInstructions );
			
			// the save image button
			this.m_btnSaveImage 	= new SaveImageButton;
			this.m_btnSaveImage.x	= DS.stage.stageWidth - this.m_btnSaveImage.width - 10.0;
			this.m_btnSaveImage.y	= DS.stage.stageHeight * 0.5 - this.m_btnSaveImage.height - 5.0;
			
			// the post to twitter button
			this.m_btnTwitter	= new PostToTwitterButton;
			this.m_btnTwitter.x	= DS.stage.stageWidth - this.m_btnTwitter.width - 10;
			this.m_btnTwitter.y = DS.stage.stageHeight * 0.5 + 5.0;
			
			// bind our keys
			Input.bind( Key.SPACE, this._onStartCountdown );
			Input.bind( Key.ESCAPE, this._onEscape );
			
			// add us to the update
			DS.addToUpdate( this );
			
			// reset everything
			this.reset();
		}
		
		/**
		 * Destroys the logic and clears it for garbage collection
		 */
		public function destroy():void
		{
			// remove from update
			DS.removeFromUpdate( this );
			
			// unbind keys
			Input.unbind( Key.SPACE );
			Input.unbind( Key.ESCAPE );
			
			// destroy any objects
			if ( this.m_timer != null )
				this.m_timer.destroy();
			if ( this.m_countdown != null )
				this.m_countdown.destroy();
			if ( this.m_gameInstructions != null )
				this.m_gameInstructions.destroy();
			if ( this.m_btnSaveImage != null )
				this.m_btnSaveImage.destroy();
			if ( this.m_btnTwitter != null )
				this.m_btnTwitter.destroy();
			for each( var t:Track in this.m_tracks )
				t.destroy();
				
			// null our properties
			this.m_gameInstructions			= null;
			this.m_timer					= null;
			this.m_countdown				= null;
			this.m_tracks					= null;
			this.m_firstHumanFinisherTrack	= null;
			this.m_btnSaveImage				= null;
			this.m_btnTwitter				= null;
		}
		
		/**
		 * Resets the game
		 */
		public function reset():void
		{
			// reset all our objects
			for each( var t:Track in this.m_tracks )
				t.reset();
			this.m_timer.reset();
			this.m_countdown.reset();
			this.m_gameInstructions.reset();
			if ( this.m_btnSaveImage.parent != null )
				this.m_btnSaveImage.parent.removeChild( this.m_btnSaveImage );
			if ( this.m_btnTwitter.parent != null )
				this.m_btnTwitter.parent.removeChild( this.m_btnTwitter );
			
			// reset our vars
			this.m_isGameStarted			= false;
			this.m_isGameStopped			= false;
			this.m_hasGameJustStopped		= false;
			this.m_numRunnersFinished		= 0;
			this.m_numHumansFinished		= 0;
			this.m_numHumansInjured			= 0;
			this.m_lastPosX					= 0.0;
			this.m_firstHumanFinisherNumber	= null;
			this.m_firstHumanFinisherTrack	= null;
			
			this.m_gameInstructions.show( "Press SPACE to start the countdown" );
		}
		
		/**
		 * Called every frame the Logic is active
		 * @param dt The delta time
		 */
		public function update( dt:Number ):void
		{
			// find the winning track and how much to move
			var runnerX:Number		= 0.0;
			var moveX:Number 		= 0.0;
			var winningX:Number		= 0.0;
			var winningY:Number 	= 0.0;
			var winningTrack:Track	= null;
			for each( var track:Track in this.m_tracks )
			{					
				// the position of the runners (winning track)
				var rx:Number	= track.runner.position.x;
				var ry:Number	= track.runner.position.y;
				if ( rx > runnerX || track.finishPos == 1 )
				{
					winningX 		= rx + track.x;
					winningY		= ry + track.y;
					runnerX			= rx;
					winningTrack	= track;
				}
				
				// to move the camera
				if ( rx > 250.0 - track.startingX && rx > moveX )
					moveX = rx;
			}
			
			// set the winning track
			for each( track in this.m_tracks )
				track.isWinning = ( track == winningTrack );
			
			// set the winning line
			Track.winningLinePos = new Point( winningX, winningY );
				
			// get the difference to move the camera
			if ( this.m_lastPosX == 0.0 )
			{
				this.m_lastPosX = moveX;
				return;
			}
			
			var diff:Number = moveX - this.m_lastPosX;
			for each( track in this.m_tracks )
				track.x -= diff;
			this.m_lastPosX = moveX;
		}
		
		/*******************************************************************************************/
		
		// called when a runner has jumped the gun
		private function _onRunnerFalseStarted( track:Track ):void
		{
			DS.log( this, "Jumped the gun" );
			if( this.m_numHumans == 1 )
				DS.log( this, "You jumped the gun!" );
			else
				DS.log( this, track.runner/*.playerNumber*/ + " jumped the gun!" );
			this.m_waitTime = 2000;
			this._stopLogic();
		}
	
		// called when a runner has just been injured
		private function _onRunnerInjured( track:Track ):void
		{
			track.isReady = false;
			this.m_numHumansInjured++;
			
			// log it
			if( this.m_numHumans == 1 )
				DS.log( this, "You injured yourself!" );
			else
				DS.log( this, track.runner/*.playerNumber*/ + " injured themselves!" );
				
			// stop the logic
			if ( this.m_numHumans == 1 || this.m_numHumansInjured == this.m_numHumans )
			{
				this.m_waitTime = 2000;
				this._stopLogic();
				for each( var track:Track in this.m_tracks )
				{
					if ( !track.runner.isInjured && !track.runner.isFinished )
						track.runner.queueAnim( Anims.FINISH );
				}
			}
		}
		
		// called when a runner has finished
		private function _onRunnerFinished( t:Track ):void
		{
			DS.log( this, "Runner finished" );
			
			// stop the timer
			this.m_timer.stopDisplay();
			
			// store the number of runners finished and set the finish pos on the track
			this.m_waitTime = 5000;
			this.m_numRunnersFinished++;
			if ( !t.runner.isAI )
				this.m_numHumansFinished++;
			t.finishPos = this.m_numRunnersFinished;
			
			// hold the players name and pos if it's a human
			if ( !t.runner.isAI && this.m_firstHumanFinisherNumber == null )
			{
				this.m_firstHumanFinisherTrack	 	= t;
				this.m_firstHumanFinisherNumber 	= t.runner.toString()/*.playerNumber*/;
				this.m_firstHumanFinisherPos 		= t.finishPos;
				
				// show the save image button
				DS.canvas.addChild( this.m_btnSaveImage );
				DS.canvas.addChild( this.m_btnTwitter );
			}
			
			// if it's the player that finished
			if ( ( this.m_numHumansFinished + this.m_numHumansInjured ) == this.m_numHumans )
			{
				if ( this.m_firstHumanFinisherPos > 3 )
					DS.log( this, "Hard luck!" );
				else
				{
					if ( this.m_numHumans == 1 )
						DS.log( this, "Congratulations!" );
					else
						DS.log( this, "Congratulations " + this.m_firstHumanFinisherNumber + "!" );
				}
			}
				
			// if everybody's finished
			if ( ( this.m_numRunnersFinished + this.m_numHumansInjured ) == this.m_numRunners )
			{
				this._stopLogic();
				if ( this.m_firstHumanFinisherPos != 1 )
					DS.log( this, "Finish first to submit your score!" );
			}
		}
		
		// stops the logic
		private function _stopLogic():void
		{
			this.m_timer.stop();
			this.m_countdown.reset();
			this.m_gameInstructions.reset();
			for each( var t:Track in this.m_tracks )
				t.isReady = false;
			this.m_isGameStopped		= true;
			this.m_hasGameJustStopped	= true;
			DS.callLater( this._giveControl, this.m_waitTime );
		}
		
		// gives control back to the logic
		private function _giveControl():void
		{
			this.m_hasGameJustStopped = false;
			this.m_gameInstructions.show( "Press SPACE to try again" );
		}
		
		// called when the countdown is ready position
		private function _onCountdownReady():void
		{
			for each( var t:Track in this.m_tracks )
				t.runner.queueAnim( Anims.READY );
		}
		
		// called when the countdown is in the set position
		private function _onCountdownSet():void
		{
			for each( var t:Track in this.m_tracks )
				t.runner.queueAnim( Anims.SET );
		}
		
		// called when the countdown is in the go position
		private function _onCountdownGo():void
		{
			this.m_timer.start();
			var humanNumbers:Vector.<String> 	= new Vector.<String>;
			var humanKeys:Vector.<Key> 			= new Vector.<Key>;
			
			for each( var t:Track in this.m_tracks )
			{
				if ( !t.runner.isAI )
				{
					humanNumbers.push( t.runner.toString()/*.playerNumber*/ );
					humanKeys.push( t.runner.controlKey );
				}
				t.go = true;
				t.runner.queueAnim( Anims.GO );
			}
			
			// display our instructions
			var key:String = null;
			if ( this.m_numHumans == 1 )
			{
				key = humanKeys[0].keyName;
				DS.debug( this, "Hold " + key + " to run" );
			}
			else
			{
				var display:String 	= "";
				var len:int 		= humanNumbers.length;
				for ( var i:int = 0; i < len; i++ )
				{
					key 	= humanKeys[i].keyName;
					display += "[" + humanNumbers[i] + "] Hold " + key + " to run";
					if ( i < len - 1 )
						display += ", ";
				}
				DS.debug( this, display );
			}
		}
		
		// called when we press to start the countdown
		private function _onStartCountdown():void
		{
			// reset
			if ( this.m_isGameStopped )
			{
				if ( this.m_hasGameJustStopped )
					return;
				this.reset();
				return;
			}
			
			// if we've started, do nothing
			if ( this.m_isGameStarted )
				return;
				
			// start the countdown
			this.m_countdown.start();
			this.m_gameInstructions.reset();
			this.m_isGameStarted = true;
			for each( var t:Track in this.m_tracks )
				t.isReady = true;
		}
		
		// called when we press the escape button
		private function _onEscape():void
		{
			trace( "WE WANT TO ESCAPE" );
		}
		
	}
	
}