package com.divillysausages.sprint.logic
{
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.input.Input;
	import com.divillysausages.ds.input.Key;
	import com.divillysausages.ds.security.SafeNumber;
	import com.divillysausages.sprint.anim.Anim;
	import com.divillysausages.sprint.anim.AnimFrameInfo;
	import com.divillysausages.sprint.anim.AnimManager;
	import com.divillysausages.sprint.anim.AnimParams;
	import com.divillysausages.sprint.anim.AnimParser;
	import com.divillysausages.sprint.assets.Resources;
	import com.divillysausages.sprint.constants.Anims;
	import com.divillysausages.sprint.constants.Constants;
	import com.divillysausages.sprint.gui.FalseStart;
	import com.divillysausages.sprint.gui.FinishingPosition;
	import com.divillysausages.sprint.gui.Gauge;
	import com.divillysausages.sprint.gui.Injury;
	import com.divillysausages.sprint.gui.PersonalBest;
	import com.divillysausages.sprint.gui.PostToTwitterButton;
	import com.divillysausages.sprint.gui.RunnerTime;
	import com.divillysausages.sprint.gui.WorldRecord;
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.geom.Point;
	import org.osflash.signals.Signal;
	
	public class Runner extends Sprite implements IUpdateObj
	{
		
		/*******************************************************************************************/
		
		private static var m_numPlayers:int = 0; // the number of human players
		
		/*******************************************************************************************/
		
		/**
		 * The signal dispatched when we've finished the race. It should take one parameter of type Runner
		 */
		public var signalOnFinished:Signal = null;
		
		/**
		 * The signal dispatched when we've started too early. It should take one parameter of type Runner
		 */
		public var signalOnStartTooEarly:Signal = null;
		
		/**
		 * The signal dispatched when we've been injured. It should take one parameter of type Runner
		 */
		public var signalOnInjured:Signal = null;
		
		/*******************************************************************************************/
		
		private var m_track:Track 					= null;	// the track that we're running on
		private var m_pos:Point 					= new Point;	// our position
		private var m_gauge:Gauge 					= null;	// the gauge that shows our effort
		private var m_speed:SafeNumber 				= new SafeNumber( 235.0 );	// our speed
		private var m_gaugeValue:SafeNumber 		= new SafeNumber( 0.0 );	// our current gauge value
		private var m_gaugeSpeed:SafeNumber 		= new SafeNumber( 1.0 );	// the speed that our gauge fills at
		private var m_finishTime:RunnerTime 		= null;	// the time that shows up after we finished
		private var m_finishPos:FinishingPosition 	= null;	// the position that we finish in
		//private var m_runnerNumber:RunnerNumber 	= null;	// the runner number (if we're human)
		private var m_injury:Injury 				= null;	// the injury object, if we hurt ourselves
		private var m_falseStart:FalseStart 		= null;	// the false start, if we go too early
		private var m_personalBest:PersonalBest 	= null;	// the personal best marker, if we get a good time
		private var m_worldRecord:WorldRecord		= null;	// the world record marker, if we get it
		//private var m_controls:Controls 			= null;	// the controls that show which button to hit
		private var m_animHolder:Bitmap				= null;	// where our anim is going to place its frames
		private var m_animString:String 			= null;	// the string that we use to get our animations
		private var m_animID:String					= null; // the id that we use when playing/stopping animations
		private var m_isRunning:Boolean 			= false;// are we currently running?
		private var m_isReady:Boolean 				= false;// are we ready?
		private var m_canRun:Boolean				= false;// are we in the set animation?
		private var m_isAI:Boolean 					= true;	// are we ai?
		private var m_isFinished:Boolean 			= false;// are we finished?
		private var m_isActive:Boolean				= true;// is the runner active?
		private var m_hasFalseStarted:Boolean		= false;// have we false started?
		private var m_aiGaugeLimit:Number 			= 0.0;	// the limit before the ai stops accelerating
		private var m_falseStartCurrTime:Number		= 0.0;	// the current time if it's a false start (so we run a bit)
		private var m_falseStartTime:Number		 	= 0.5;	// the max time to run if it's a false start
		private var m_controlKey:Key				= null;	// the control key that we're using to run
		
		/*******************************************************************************************/
		
		/**
		 * Is the Runner active?
		 */
		public function get active():Boolean { return this.m_isActive; }
		
		/**
		 * Set the control key that we're using to run
		 */
		public function get controlKey():Key { return this.m_controlKey; }
		public function set controlKey( k:Key ):void
		{
			this.m_controlKey = k;
			//this.m_controls = new Controls;
			//this.m_controls.Key = this.m_controlKey;
		}
		
		/**
		 * Set if we're ready
		 */
		public function set isReady( b:Boolean ):void
		{
			this.m_isReady = b;
		}
		
		/**
		 * Sets if the runner is ai or not
		 */
		public function get isAI():Boolean { return this.m_isAI; }
		public function set isAI( b:Boolean ):void
		{
			this.m_isAI = b;
			if ( this.m_isAI )
			{
				this.m_animString = "ai";
				this._initAI(); // basically removes any human stuff
			}
			else
			{
				this.m_animString = ( Runner.m_numPlayers == 0 ) ? "player1" : "player2";
				Runner.m_numPlayers++;
				this._initHuman();
			}
			
			// set the default animation
			// TODO: set up the params for the animation
			this.m_animID = this.m_animString + Math.random();
			this.queueAnim( Anims.STAND );
		}
		
		/**
		 * Returns the position of the runner
		 */
		public function get position():Point { return this.m_pos; }
		
		/**
		 * The finishing position of the runner
		 */
		public function get finishPos():int { return this.m_finishPos.pos; }
		public function set finishPos( i:int ):void
		{
			this.m_finishPos.pos = i;
		}
		
		/**
		 * The player number of the runner, for display
		 */
		//public function get playerNumber():String { return ( this.m_runnerNumber != null ) ? this.m_runnerNumber.num : ""; }
		//public function set playerNumber( s:String ):void
		//{
			//if ( this.m_runnerNumber != null )
				//this.m_runnerNumber.num = s;
		//}
		
		/**
		 * Is the player injured?
		 */
		public function get isInjured():Boolean { return ( this.m_injury != null ) ? this.m_injury.visible : false; }
		
		/**
		 * Is the player finished?
		 */
		public function get isFinished():Boolean { return this.m_isFinished; }
		
		/*******************************************************************************************/
		
		/**
		 * Creates a new Runner
		 * @param t The track that we belong to
		 */
		public function Runner( t:Track ):void
		{
			this.m_track = t;
			
			// create our gauge
			this.m_gauge 		= new Gauge;
			this.m_gauge.x		= -this.m_gauge.width - 28;
			this.m_gauge.y		= -this.m_gauge.height + 12;
			this.m_gauge.signalOnOverLimit.add( this._onGaugeOverLimit );
			this.addChild( this.m_gauge );
			
			// create the finish time display
			this.m_finishTime 	= new RunnerTime;
			this.m_finishTime.x	= 16.0;
			this.m_finishTime.y = -18.0;
			this.addChild( this.m_finishTime );
			
			// create the finish pos display
			this.m_finishPos 	= new FinishingPosition;
			this.m_finishPos.x	= 80.0;
			this.m_finishPos.y	= -12;
			this.addChild( this.m_finishPos );
			
			// create the personal best
			this.m_personalBest 	= new PersonalBest;
			this.m_personalBest.y	= -24.0;
			this.addChild( this.m_personalBest );
			
			// create the world record
			this.m_worldRecord 		= new WorldRecord;
			this.m_worldRecord.y	= -24.0;
			this.addChild( this.m_worldRecord );
			
			// it's not active
			this.mouseChildren	= false;
			this.mouseEnabled 	= false;
			
			// create our anim holder
			this.m_animHolder = new Bitmap;
			this.addChild( this.m_animHolder );
			
			// create our signals
			this.signalOnFinished		= new Signal( Runner );
			this.signalOnInjured		= new Signal( Runner );
			this.signalOnStartTooEarly	= new Signal( Runner );
			
			// add us to the update
			DS.addToUpdate( this );
		}
		
		/**
		 * Destroys the Runner and clears it for garbage collection
		 */
		public function destroy():void
		{
			// remove from update
			DS.removeFromUpdate( this );
			
			// stop any anims
			AnimManager.instance.stopAnims( this.m_animID );
			
			// clear our signal
			this.signalOnFinished.removeAll();
			this.signalOnInjured.removeAll();
			this.signalOnStartTooEarly.removeAll();
				
			// graphics
			this.removeChild( this.m_animHolder );
			if ( this.parent != null )
				this.parent.removeChild( this );
				
			// destroy anything that needs to be destroyed
			if ( this.m_gauge != null )			this.m_gauge.destroy();
			if ( this.m_finishTime != null )	this.m_finishTime.destroy();
			if ( this.m_finishPos != null )		this.m_finishPos.destroy();
			//if ( this.m_runnerNumber != null )	this.m_runnerNumber.destroy();
			if ( this.m_injury != null )		this.m_injury.destroy();
			if ( this.m_falseStart != null )	this.m_falseStart.destroy();
			if ( this.m_personalBest != null )	this.m_personalBest.destroy();
			if ( this.m_worldRecord != null )	this.m_worldRecord.destroy();
			//if ( this.m_controls != null )		this.m_controls.destroy();
				
			// null our params
			this.signalOnFinished		= null;
			this.signalOnInjured		= null;
			this.signalOnStartTooEarly	= null;
			this.m_animHolder			= null;
			this.m_track				= null;
			this.m_pos					= null;
			this.m_gauge				= null;
			this.m_speed				= null;
			this.m_gaugeValue			= null;
			this.m_gaugeSpeed			= null;
			this.m_finishTime			= null;
			this.m_finishPos			= null;
			//this.m_runnerNumber		= null;
			this.m_injury				= null;
			this.m_falseStart			= null;
			this.m_personalBest			= null;
			this.m_worldRecord			= null;
			//this.m_controls			= null;
				
			// reset the number of players on the class
			Runner.m_numPlayers = 0;
		}
		
		/**
		 * Resets the runner so it can be reused
		 */
		public function reset():void
		{
			this.m_pos.setTo( 0.0, 0.0 );		// reset our position
			this.m_gauge.value			= 0.0;	// clear our gauge
			this.m_gaugeValue.value		= 0.0;	// clear our gauge value
			this.m_finishTime.time		= null;	// clear our finish time
			this.m_finishPos.pos		= -1;	// clear our finish pos
			this.m_isReady				= false;// we're not ready
			this.m_isFinished			= false;// we're not finished
			this.m_isRunning			= false;// we're not running
			this.m_hasFalseStarted		= false;// we haven't false started
			this.m_canRun				= false;// we can't run
			this.m_falseStartCurrTime	= 0.0;	// our false start current time
			this._calculateAILimit();		// calculate our ai limit
			if ( this.m_injury != null )
				this.m_injury.reset();		// reset our injury
			if ( this.m_falseStart != null )
				this.m_falseStart.reset();	// reset our false start
			if ( this.m_personalBest != null )
				this.m_personalBest.reset();	// reset our personal best
			if ( this.m_worldRecord != null )
				this.m_worldRecord.reset();		// reset our world record
			//if ( this.m_controls != null )
				//this.m_controls.reset();		// reset our controls
				
			// reset our pos
			this.x = this.m_track.x;
			this.y = this.m_track.y;
				
			// play our stand anim
			AnimManager.instance.stopAnims( this.m_animID );
			this.queueAnim( Anims.STAND );
		}
		
		/**
		 * Plays an anim on the Runner
		 * @param anim The anim to play
		 */
		public function playAnim( anim:String ):void
		{
			// set if we can run
			if ( !this.m_canRun )
				this.m_canRun = ( anim == Anims.SET );
				
			// set if we should loop and finish the anim name
			var shouldLoop:Boolean	= ( anim == Anims.RUN || anim == Anims.WIN );
			anim 					= anim + "_" + this.m_animString;	// add our anim string
			
			// create our anim params
			var params:AnimParams 	= AnimManager.instance.checkOutAnimParams();
			params.bmdOffsets		= AnimParser.instance.getOffsets( anim );
			params.bmds				= AnimParser.instance.getFrames( anim );
			params.target			= this.m_animHolder;
			params.onFrame			= this._onAnimFrame;
			params.tps				= Constants.ANIM_FPS;
			params.loops			= ( shouldLoop ) ? int.MAX_VALUE : 0;
			
			// get our frames
			var frameInfos:Vector.<AnimFrameInfo> = AnimParser.instance.getAnimFrameInfos( anim );
			
			// play the anim
			var a:Anim = AnimManager.instance.playAnim( this.m_animID, params, false );
			if ( frameInfos.length > 0 )
			{
				for each( var afi:AnimFrameInfo in frameInfos )
					a.addFrameInfo( afi.name, afi.start, afi.end );
			}
		}
		
		/**
		 * Queues an animation to play
		 * @param anim The anim to queue
		 */
		public function queueAnim( anim:String ):void
		{
			var a:Anim = AnimManager.instance.getAnim( this.m_animID );
			if ( a != null )
				a.params.loops = 0; // kill the loops so it stops at the end of the anim
			this.playAnim( anim );
		}
		
		/**
		 * Called every frame the runner is active
		 * @param dt The delta time
		 */
		public function update( dt:Number ):void
		{
			// if we have a control key, check if it's pressed
			if ( this.m_controlKey != null )
				this.m_isRunning = Input.isPressed( this.m_controlKey );
				
			// run for a bit if we false start
			if ( this.m_falseStart != null && this.m_falseStart.visible )
			{
				if ( this.m_falseStartCurrTime > this.m_falseStartTime )
					return; // the player can do nothing
				else
					this.m_falseStartCurrTime += dt;
			}
			
			// if we can't run, do nothing
			if ( !this.m_canRun )
				return;
			
			// if we're running and we're not ready, then it's a false start
			if ( this.m_track.isReady && this.m_isRunning && !this.m_isReady && this.m_canRun )
			{
				if ( this.m_falseStart != null )
					this.m_falseStart.show();
				this.m_hasFalseStarted = true;
				this.signalOnStartTooEarly.dispatch( this );
			}
			
			// move our value between 0.0 and 1.0
			if ( this.m_isAI )
			{
				var limit:Number = this.m_aiGaugeLimit + Math.random() * 0.09;
				if ( this.m_gaugeValue.value < limit && this.m_isReady )
					this.m_gaugeValue.value += this.m_gaugeSpeed.value * dt;
				else
					this.m_gaugeValue.value -= this.m_gaugeSpeed.value * dt;
			}
			else
			{
				if ( ( this.m_isRunning && this.m_track.isReady ) ||
					 ( this.m_falseStart != null && this.m_falseStart.visible ) )
					this.m_gaugeValue.value += this.m_gaugeSpeed.value * dt;
				else if ( !this.m_track.isReady )
					this.m_gaugeValue.value -= this.m_gaugeSpeed.value * dt * 0.75;
				else
					this.m_gaugeValue.value -= this.m_gaugeSpeed.value * dt;
			}
			
			// clamp if
			if ( this.m_gaugeValue.value < 0.0 ) this.m_gaugeValue.value = 0.0;
			else if ( this.m_gaugeValue.value > 1.0 ) this.m_gaugeValue.value = 1.0;
			
			// get our actual value from our curve, something like (-((x+0.1)*10)^-1 + 1)/0.98
			var x:Number = this.m_gaugeValue.value;
			x += 0.1;
			x *= 10.0;
			x = -1.0 / x;
			x += 1.0;
			x /= 0.98;
			this.m_gauge.value = x;
			
			// move us
			var move:Number = this.m_speed.value * this.m_gauge.value * dt;
			this.m_pos.x 	+= move;
			this.x			= this.m_pos.x + this.m_track.x;
			this.y			= this.m_pos.y + this.m_track.y;
			
			// update all the other positions
			this._updatePositions();
			
			// check if we're finished
			if ( this.m_pos.x > this.m_track.length && this.m_track.isReady )
			//if ( this.m_pos.x > 200 && this.m_track.isReady )
			{
				// set the finish time
				this.m_track.saveFinishTime();
				this.m_finishTime.time = this.m_track.time;
				
				// set that we've finished
				this.m_isFinished	= true;
				this.m_isReady		= false;
				//if ( this.m_controls != null )
					//this.m_controls.visible = false;
				this.signalOnFinished.dispatch( this );
				
				// check if we've beaten our personal best
				var pb:Number 	= GameSave.instance.getPBForPlayer( this.m_track.id );
				var time:Number	= this.m_track.timeNumber;
				if ( time < pb )
				{
					// store the new pb
					DS.log( this, "Player " + this.m_track.id + " just did a pb of " + time );
					GameSave.instance.setPBForPlayer( this.m_track.id, time );
					
					// show our graphic
					if ( this.finishPos == 4 )
						this.m_personalBest.x = this.m_finishTime.x + this.m_finishTime.width + 24;
					else
						this.m_personalBest.x = this.m_finishPos.x + this.m_finishPos.width;
					this.m_personalBest.show();
				}
				else
					DS.debug( this, "Player " + this.m_track.id + "'s time of " + time + " didn't beat their pb of " + pb );
					
				// check if we've beaten the world record
				var wr:Number = GameSave.instance.worldRecord;
				if ( time < wr )
				{
					// store the new world record
					DS.log( this, "Player " + this.m_track.id + " just did a wr of " + time );
					GameSave.instance.worldRecord = time;
					
					// show our graphic
					this.m_worldRecord.x = this.m_personalBest.x + this.m_personalBest.width - 10;
					this.m_worldRecord.show();
				}
				else
					DS.debug( this, "Player " + m_track.id + "'s time of " + time + " didn't beat the world record of " + wr );
					
				// if we're the player, set the twitter text
				if ( !this.isAI )
				{
					var btn:PostToTwitterButton = ( DS.main as Main ).logic.btnTwitter;
					btn.setText( this.m_track.time, ( time < pb ), ( time < wr ) );
				}
			}
		}
		
		/*******************************************************************************************/
		
		// inits the runner for a human
		private function _initHuman():void
		{
			// create our false start
			this.m_falseStart 	= new FalseStart;
			this.m_falseStart.y	= -24;
			this.addChild( this.m_falseStart );
			
			// create the injury
			this.m_injury 	= new Injury;
			this.m_injury.x	= 12;
			this.addChild( this.m_injury );
			
			// create the runner number
			//this.m_runnerNumber = new RunnerNumber;
			
			// show the gauge
			this.m_gauge.visible = true;
		}
		
		// inits the runner for an ai
		private function _initAI():void
		{
			// destroy our false start
			if ( this.m_falseStart != null )
			{
				this.m_falseStart.destroy();
				this.m_falseStart = null;
			}
			
			// destroy our injury
			if ( this.m_injury != null )
			{
				this.m_injury.destroy();
				this.m_injury = null;
			}
			
			// destroy our runner number
			//if ( this.m_runnerNumber != null )
			//{
				//this.m_runnerNumber.destroy();
				//this.m_runnerNumber = null;
			//}
			
			// hide the gauge
			this.m_gauge.visible = false;
		}
		
		// called when our gauge goes over the limit
		private function _onGaugeOverLimit():void
		{
			if ( this.m_injury != null )
				this.m_injury.show();
			this.signalOnInjured.dispatch( this );
		}
		
		// updates the positions on all our objects
		private function _updatePositions():void
		{
			//if ( this.m_controls != null ) 	this.m_controls.position = this.m_pos;
			//if ( this.m_runnerNumber != null ) this.m_runnerNumber.position = this.m_pos;
		}
		
		// calculates the ai limit (over this, they accelerate, under it, they decelerate)
		private function _calculateAILimit():void
		{
			this.m_aiGaugeLimit = 0.32 + ( Math.random() * 0.13 );
		}
		
		// called when our animation passes a named frame
		private function _onAnimFrame( frame:String ):void
		{
			if ( frame == Anims.FRAME_FOOT_HIT && !this.m_isAI )
				DS.soundManager.playSound( Resources.SFX_RUN );
			else if ( frame == Anims.FRAME_END_GO && !this.m_hasFalseStarted )
				this.queueAnim( Anims.RUN );
			else if ( frame == Anims.FRAME_END_STOP && this.finishPos == 1 )
				this.queueAnim( Anims.WIN );
			else if ( frame == Anims.FRAME_RIGHT_FOOT && this.isInjured )
				this.queueAnim( Anims.INJURED );
		}
	}

}