package com.divillysausages.sprint.logic
{
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.security.SafeNumber;
	import com.divillysausages.sprint.constants.Anims;
	import com.divillysausages.sprint.constants.Constants;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.geom.Point;
	import org.osflash.signals.Signal;
	
	public class Track extends Sprite
	{
		
		/*******************************************************************************************/
		
		// the indicies for the track bmds
		private static const M_IDX_NORMAL:int		= 0;
		private static const M_IDX_WINNING:int 		= 1;
		private static const M_IDX_END_NORMAL:int	= 2;
		private static const M_IDX_END_WINNING:int	= 3;
		private static const M_IDX_OFF_TRACK:int	= 4;
		
		/*******************************************************************************************/
		
		private static var m_winningLine:Shape 			= null; // the winning line shape to show who's currently winning
		private static var m_bmds:Vector.<BitmapData>	= null;
		
		/*******************************************************************************************/
		
		/**
		 * The winning line shape
		 */
		public static function get winningLine():Shape { return Track.m_winningLine; }
		
		/**
		 * The position for the winning line
		 */
		public static function set winningLinePos( p:Point ):void
		{
			Track.m_winningLine.x = p.x;
			Track.m_winningLine.y = p.y;
		}
		
		/*******************************************************************************************/
		
		/**
		 * The signal dispatched when our runner is injured. It should take one parameter of type Track
		 */
		public var signalOnRunnerInjured:Signal = null;
		
		/**
		 * The signal dispatched when our runner has finished. It should take one parameter of type Track
		 */
		public var signalOnRunnerFinished:Signal = null;
		
		/**
		 * The signal dispatched when our runner has false started. It should take one parameter of type Track
		 */
		public var signalOnRunnerFalseStart:Signal = null;
		
		/*******************************************************************************************/
		
		private var m_logic:Logic 				= null;
		private var m_position:Point			= new Point; // our position
		private var m_bitmaps:Vector.<Bitmap>	= null; // our track images
		private var m_width:SafeNumber			= new SafeNumber( 48.0 );	// the width of the track
		private var m_length:SafeNumber			= new SafeNumber( 2000.0 );// the length of the track
		private var m_numSections:int			= 10;	// the number of sections in the track
		private var m_runner:Runner 			= null;	// the runner on the track
		private var m_isFinished:Boolean 		= false;// has our runner finished?
		private var m_hasJumpedTheGun:Boolean 	= false;// has our runner jumped the gun?
		private var m_isInjured:Boolean 		= false;// is our runner injured?
		private var m_isReady:Boolean 			= false;// are we ready?
		private var m_isWinning:Boolean 		= false;// are we winning?
		private var m_id:int 					= 0;	// the id for this track
		private var m_startingX:Number 			= 0.0;	// the starting x position for this track
		private var m_finishTime:SafeNumber 	= new SafeNumber( 0 ); // the finish time for our runner
		
		/*******************************************************************************************/
		
		/**
		 * The runner that's on this track
		 */
		public function get runner():Runner { return this.m_runner; }
		
		/**
		 * The track's position
		 */
		public function get position():Point { return this.m_position; }
		
		/**
		 * The width of the track
		 */
		override public function get width():Number { return this.m_width.value; }
		
		/**
		 * The length of the track
		 */
		public function get length():Number { return this.m_length.value; }
		
		/**
		 * Is this track ready?
		 */
		public function get isReady():Boolean { return this.m_isReady; }
		public function set isReady( b:Boolean ):void
		{
			this.m_isReady = b;
			if ( !this.m_isReady )
				this.m_runner.isReady = this.m_isReady;
		}
		
		/**
		 * Tells the track to go
		 */
		public function set go( b:Boolean ):void
		{
			this.m_runner.isReady = b;
		}
		
		/**
		 * The current time, as a string
		 */
		public function get time():String { return this.m_logic.time; }
		
		/**
		 * The current time, as a number
		 */
		public function get timeNumber():Number { return this.m_logic.timeNumber; }
		
		/**
		 * The current finish time
		 */
		public function get finishTime():Number { return int( this.m_finishTime.value ); }
		
		/**
		 * The ID of this track
		 */
		public function get id():int { return this.m_id; }
		
		/**
		 * The total number of runners in the game
		 */
		public function get numRunners():int { return this.m_logic.numRunners; }
		
		/**
		 * The finishing position for our runner
		 */
		public function get finishPos():int { return this.m_runner.finishPos; }
		public function set finishPos( i:int ):void
		{
			this.m_runner.finishPos = i;
		}
		
		/**
		 * The starting x for the track
		 */
		public function get startingX():Number { return this.m_startingX; }
		
		/**
		 * Is this currently the track that's winning?
		 */
		public function get isWinning():Boolean { return this.m_isWinning; }
		public function set isWinning( b:Boolean ):void
		{
			if ( this.m_isWinning != b )
			{
				this.m_isWinning = b;
				this._fillGraphics();
			}
		}
		
		/*******************************************************************************************/
		
		/**
		 * Creates a new Track
		 * @param logic The main logic class
		 * @param id The ID of this class
		 */
		public function Track( logic:Logic, id:int ):void
		{
			this.m_logic	= logic;
			this.m_id		= id;
			
			// create our track if needed
			if ( Track.m_bmds == null )
				this._createTrack();
				
			// create our winning line if needed
			if ( Track.m_winningLine == null )
				this._createWinningLine();
				
			// fill our graphics
			this.m_bitmaps 		= new Vector.<Bitmap>( this.m_numSections + 7, true );
			var xDiff:Number	= this.m_length.value / this.m_numSections;
			var hw:Number		= this.m_width.value * 0.5;
			for ( var i:int = 0; i < this.m_numSections + 7; i++ )
			{
				var b:Bitmap 		= new Bitmap;
				b.x					= -xDiff * 2 + xDiff * i - hw;
				b.y					= -hw;
				this.m_bitmaps[i]	= b;
				this.addChild( b );
			}
			this._fillGraphics();
			
			// create our signals
			this.signalOnRunnerInjured 		= new Signal( Track );
			this.signalOnRunnerFinished		= new Signal( Track );
			this.signalOnRunnerFalseStart	= new Signal( Track );
			
			// no clicking
			this.mouseChildren	= false;
			this.mouseEnabled	= false;
		}
		
		/**
		 * Destroys the track and clears it for garbage collection
		 */
		public function destroy():void
		{
			// destroy the winning line
			if ( Track.m_winningLine != null )
			{
				if ( Track.m_winningLine.parent != null )
					Track.m_winningLine.parent.removeChild( Track.m_winningLine );
				Track.m_winningLine = null;
			}
			
			// clear our signals
			this.signalOnRunnerInjured.removeAll();
			this.signalOnRunnerFinished.removeAll();
			this.signalOnRunnerFalseStart.removeAll();
			
			// destroy our running
			if ( this.m_runner != null )
				this.m_runner.destroy();
				
			// graphics
			this.removeChildren();
			if ( this.parent != null )
				this.parent.removeChild( this );
				
			// null our properties
			this.signalOnRunnerInjured		= null;
			this.signalOnRunnerFinished		= null;
			this.signalOnRunnerFalseStart	= null;
			this.m_bitmaps					= null;
			this.m_logic					= null;
			this.m_runner					= null;
		}
		
		/**
		 * Resets the track
		 */
		public function reset():void
		{			
			// reset our properties
			this.m_isFinished	 	= false;
			this.m_hasJumpedTheGun	= false;
			this.m_isInjured		= false;
			this.m_isReady			= false;
			this.isWinning			= false;
			this.m_finishTime.value	= 0.0;
			
			// get the total height
			var w:Number		= this.m_width.value;
			var totalH:Number 	= w * this.numRunners;
			var startY:Number	= DS.stage.stageHeight * 0.5 + totalH * 0.5 - w * 0.5;
			this.m_startingX	= 50.0 + ( w * this.m_id );
			this.x				= this.m_startingX;
			this.y				= startY - ( w * this.m_id );
			this.m_position.setTo( this.x, this.y );
			
			// reset our runner
			this.m_runner.reset();
		}
		
		/**
		 * Creates a new runner for this track
		 */
		public function createRunner():void
		{
			this.m_runner = new Runner( this );
			this.m_runner.signalOnFinished.add( this._onRunnerFinished );
			this.m_runner.signalOnInjured.add( this._onRunnerInjured );
			this.m_runner.signalOnStartTooEarly.add( this._onRunnerJumpedTheGun );
		}
		
		/**
		 * Save the current finish time
		 */
		public function saveFinishTime():void
		{
			this.m_finishTime.value = this.timeNumber;
		}
		
		/*******************************************************************************************/
		
		// creates the track
		private function _createTrack():void
		{
			// create all the parts
			Track.m_bmds 							= new Vector.<BitmapData>( 5, true );
			Track.m_bmds[Track.M_IDX_NORMAL]		= this._drawTrackPart( false, false, false );
			Track.m_bmds[Track.M_IDX_WINNING]		= this._drawTrackPart( true, false, false );
			Track.m_bmds[Track.M_IDX_END_NORMAL]	= this._drawTrackPart( false, true, false );
			Track.m_bmds[Track.M_IDX_END_WINNING]	= this._drawTrackPart( true, true, false );
			Track.m_bmds[Track.M_IDX_OFF_TRACK]		= this._drawTrackPart( false, false, true );
		}
		
		// draws the graphics for the track
		private function _drawTrackPart( isWinning:Boolean, isEnd:Boolean, isOff:Boolean ):BitmapData
		{
			var s:Shape = new Shape;
			
			// get our width and height
			var w:Number 	= this.m_width.value;
			var l:Number 	= this.m_length.value;
			var div:Number 	= l / this.m_numSections + (( isOff ) ? 1 : 0 );
			var ps:int		= Constants.PIXEL_SIZE;
			
			// outer
			s.graphics.beginFill( 0xBA992B );
			s.graphics.moveTo( w, 0.0 );
			s.graphics.lineTo( w + div, 0.0 );
			s.graphics.lineTo( div, w );
			s.graphics.lineTo( 0.0, w );
			s.graphics.lineTo( w, 0.0 );
			s.graphics.endFill();
			
			// inner
			s.graphics.beginFill( (isOff) ? 0xcccccc : ( isWinning ) ? 0xEBCA5E : 0xD1AD34 );
			s.graphics.moveTo( w - ps, ps );
			s.graphics.lineTo( w + div - ps, ps );
			s.graphics.lineTo( div + ps, w - ps );
			s.graphics.lineTo( ps, w - ps );
			s.graphics.lineTo( w - ps, ps );
			s.graphics.endFill();
			
			// white line
			if ( !isOff )
			{
				var iters:int 	= ( isEnd ) ? 4 : 0;
				var rads:Number	= ( Math.PI * 0.5 ) / iters;
				for ( var i:int = 0; i <= iters; i++ )
				{
					var pc:Number 	= Math.sin( rads * i );
					pc				= ( pc < 0.0 ) ? 0.0 : ( pc > 1.0 ) ? 1.0 : pc;
					var value:int	= Math.ceil( ( div * pc ) / ps ) * ps - ps; // round to the nearest pixel
					s.graphics.beginFill( 0xffffff, 0.6 );
					s.graphics.moveTo( w + value, 0.0 );
					s.graphics.lineTo( w + value + ps, 0.0 );
					s.graphics.lineTo( ps + value, w );
					s.graphics.lineTo( value, w );
					s.graphics.lineTo( w + value, 0.0 );
					s.graphics.endFill();
				}
			}
			
			// draw the bitmap
			var bmd:BitmapData = new BitmapData( s.width, s.height, true, 0x00000000 );
			bmd.draw( s );
			return bmd;
		}
		
		// fills our graphics
		private function _fillGraphics():void
		{
			var len:int = this.m_bitmaps.length;
			for ( var i:int = 0; i < len; i++ )
			{
				if ( i <= 1 || i > this.m_numSections + 1 )
					this.m_bitmaps[i].bitmapData = Track.m_bmds[Track.M_IDX_OFF_TRACK];
				else if ( i == this.m_numSections + 1 )
					this.m_bitmaps[i].bitmapData = ( this.m_isWinning ) ? Track.m_bmds[Track.M_IDX_END_WINNING] : Track.m_bmds[Track.M_IDX_END_NORMAL];
				else
					this.m_bitmaps[i].bitmapData = ( this.m_isWinning ) ? Track.m_bmds[Track.M_IDX_WINNING] : Track.m_bmds[Track.M_IDX_NORMAL];
			}
		}
		
		// called when our runner has been injured
		private function _onRunnerInjured( runner:Runner ):void
		{
			if ( this.m_isInjured )
				return;
			this.m_isInjured = true;
			this.signalOnRunnerInjured.dispatch( this );
		}
		
		// called when our runner has finished
		private function _onRunnerFinished( runner:Runner ):void
		{
			if ( this.m_isFinished )
				return;
			this.m_isFinished	= true;
			this.m_isReady		= false;
			this.m_runner.queueAnim( Anims.FINISH ); // will queue the win anim if they're first
			
			// dispatch our signal
			this.signalOnRunnerFinished.dispatch( this );
		}
		
		// called when our runner has gone too early
		private function _onRunnerJumpedTheGun( runner:Runner ):void
		{
			if ( this.m_hasJumpedTheGun )
				return;
			this.m_hasJumpedTheGun = true;
			this.m_runner.queueAnim( Anims.GO );
			this.m_runner.queueAnim( Anims.FINISH );
			
			// dispatch our signal
			this.signalOnRunnerFalseStart.dispatch( this );
		}
		
		// creates the winning line
		private function _createWinningLine():void
		{
			// draw our line
			var h:Number = DS.stage.stageHeight;
			var s:Shape	 = new Shape;
			s.graphics.lineStyle( 2.0, 0x007f00, 0.5 );
			s.graphics.moveTo( -h, h );
			s.graphics.lineTo( h, -h );
			
			// store it
			Track.m_winningLine = s;
		}
		
	}

}