package com.divillysausages.sprint.states 
{
	import com.divillysausages.ds.core.GameStateManager;
	
	/**
	 * The game state manager that creates and controls all the game states
	 * @author Damian
	 */
	public class Game extends GameStateManager
	{
		
		/**
		 * The check state
		 */
		public static const STATE_CHECK:String = "check";
		
		/**
		 * The Ds logo state
		 */
		public static const STATE_DS:String = "ds";
		
		/**
		 * The hacked game state
		 */
		public static const STATE_HACKED:String = "hacked";
		
		/**
		 * The init state
		 */
		public static const STATE_INIT:String = "init";
		
		/**
		 * The instructions state
		 */
		public static const STATE_INSTRUCTIONS:String = "instructions";
		
		/**
		 * The main menu state
		 */
		public static const STATE_MAIN_MENU:String = "main_menu";
		
		/**
		 * The play state
		 */
		public static const STATE_PLAY:String = "play";
		
		/**
		 * The reset state
		 */
		public static const STATE_RESET:String = "reset";
		
		/**
		 * The results screen
		 */
		public static const STATE_RESULTS:String = "results";
		
		/**
		 * The submit score state
		 */
		public static const STATE_SUBMIT_SCORE:String = "submitScore";
		
		/*******************************************************************************************/
		
		private static var m_instance:Game = null;	// the game instance
		
		/*******************************************************************************************/
		
		/**
		 * Changes to the next state
		 * @param next	The name of the next state to change to
		 */
		public static function changeState( next:String ):void
		{
			Game.m_instance.changeState( next );
		}
		
		/**
		 * [read-only] The name of the current state
		 */
		public static function get currentState():String { return Game.m_instance.currentState.name; }
		
		/*******************************************************************************************/
		
		private var m_main:Main = null; // a reference to the main class
		
		/*******************************************************************************************/
		
		/**
		 * Creates the game state manager and all the states
		 */
		public function Game( m:Main ) 
		{
			Game.m_instance = this;
			
			// create our states
			this.addState( new StateInit( m, Game.STATE_INIT ) );
			this.addState( new StateCheck( m, Game.STATE_CHECK ) );
			this.addState( new StateDS( m, Game.STATE_DS ) );
			//this.addState( new StateGameOver( m, Game.STATE_GAME_OVER ) );
			//this.addState( new StateInstructions( m, Game.STATE_INSTRUCTIONS ) );
			this.addState( new StateMainMenu( m, Game.STATE_MAIN_MENU ) );
			this.addState( new StatePlay( m, Game.STATE_PLAY ) );
			//this.addState( new StateReset( m, Game.STATE_RESET ) );
			//this.addState( new StateResults( m, Game.STATE_RESULTS ) );
			//this.addState( new StateStart( m, Game.STATE_START ) );
			//this.addState( new StateSubmitScore( m, Game.STATE_SUBMIT_SCORE ) );
			//this.addState( new StateOverview( m, Game.STATE_OVERVIEW ) );
			this.addState( new StateHacked( m, Game.STATE_HACKED ) );
		}
		
	}

}